#!/usr/bin/python
import json
import math
import cv2
import numpy as np
import sys
import psycopg2
import psycopg2.extras
import allensdk.core.json_utilities as json
from neuron_morphology import swc
from allensdk.internal.core.lims_pipeline_module import PipelineModule

#from surrogate_strategy import prep_json

# TODO update run_python.sh to include this path
twok_dir = "/shared/bioapps/itk/itk_shared/jp2/build"
print("WARNING: adding directory to PYTHONPATH")
print("=>  %s" % twok_dir)
sys.path.append(twok_dir)
import jpeg_twok

########################################################################
# helper functions

def calculate_centroid(x, y):
    ''' Calculates the center of a polygon, using weighted averages
    of vertex locations
    '''
    assert len(x) == len(y), "Vertex arrays are of incorrect shape"
    tot_len = 0.0
    tot_x = 0.0
    tot_y = 0.0
    for i in range(len(x)):
        x0 = x[i-1]
        y0 = y[i-1]
        x1 = x[i]
        y1 = y[i]
        seg_len = math.sqrt((x1-x0)*(x1-x0) + (y1-y0)*(y1-y0))
        tot_len += seg_len
        tot_x += seg_len * x0
        tot_x += seg_len * x1
        tot_y += seg_len * y0
        tot_y += seg_len * y1
    tot_x /= 2.0 * tot_len
    tot_y /= 2.0 * tot_len
    return tot_x, tot_y

def convert_coords_str(coord_str):
    vals = coord_str.split(',')
    x = np.array(vals[0::2], dtype=float)
    y = np.array(vals[1::2], dtype=float)
    return x, y


color_table = []
color_table.append((255,  77,  77))
color_table.append((102, 102, 255))
color_table.append(( 25, 255,  25))
color_table.append((177, 166, 255))
color_table.append(( 46, 230, 230))
color_table.append((255,  77, 255))
color_table.append((128, 230,  46))
color_table.append((255, 166,  77))
color_table.append((179, 179, 179))
color_table.append(( 77, 255, 166))
color_table.append((229, 229,  46))
color_table.append((255,  51, 153))
color_table.append((166,  77, 255))
color_table.append((151, 166,  86))

color_table.append((153,   0,   0))
color_table.append((  0,  77, 153))
color_table.append((153, 153,   0))
color_table.append(( 77, 153,   0))
color_table.append((  0, 153, 153))
color_table.append(( 13, 128,  13))
color_table.append((153,  77,   0))
color_table.append((  0,   0, 153))
color_table.append((153,   0, 153))
color_table.append((  0, 179,  89))
color_table.append((102, 102, 102))
color_table.append(( 77,   0, 153))
color_table.append((153,   0,  77))
color_table.append(( 78,  89,  30))

def color_by_index(i):
    global color_table
    #
    return color_table[i % len(color_table)]

def draw_morphology(nrn, img, somax, somay, color_by_layer=False):
    global LINE_WIDTH, resolution
    #
    soma_col = (0, 0, 0)
    axon_col = (70, 130, 180)
    dend_col = (178, 34, 34)
    apical_col = (255, 127, 80)
    for c in nrn.compartment_list:
        x0 = int(c.node1.x / resolution + somax)
        x1 = int(c.node2.x / resolution + somax)
        y0 = int(c.node1.y / resolution + somay)
        y1 = int(c.node2.y / resolution + somay)
        if color_by_layer:
            color = color_table[c.node1.layer_num]
        else:
            color = soma_col
            if c.node2.t == 2:
                color = axon_col
            elif c.node2.t == 3:
                color = dend_col
            elif c.node2.t == 4:
                color = apical_col
        cv2.line(img, (x0,y0), (x1,y1), color, LINE_WIDTH)

def write_svg(svgname, jin, nrn):
    resolution = jin["resolution"]
    dx = jin["soma"]["position"][0] - nrn.soma_root().x / resolution
    dy = jin["soma"]["position"][1] - nrn.soma_root().y / resolution
    with open(svgname, "w") as f:
        #f.write('<?xml version="1.0" encoding="UTF-8" ?>\n')
        f.write('<svg xmlns="http://www.w3.org/2000/svg" version="1.1">\n')
        # soma
        soma = jin["soma"]["path"][0]
        coords = soma.split(',')
        f.write('  <polyline points="')
        for i in range(0, len(coords), 2):
            f.write('%f,%f ' % (float(coords[i]), float(coords[i+1])))
        f.write('" stroke="black" stoke-width="2" fill="none" />\n')
        for layer in jin["layers"]:
            f.write('  <polyline points="')
            path = layer["path"]
            coords = path.split(',')
            for i in range(0, len(coords), 2):
                f.write('%f,%f ' % (float(coords[i]), float(coords[i+1])))
            f.write('" stroke="black" stoke-width="2" fill="none" />\n')
        for c in nrn.compartment_list:
            try:
                color = color_table[c.node1.layer_num]
            except:
                color = (45, 67, 89)
            x0 = int(c.node1.x / resolution + dx)
            x1 = int(c.node2.x / resolution + dx)
            y0 = int(c.node1.y / resolution + dy)
            y1 = int(c.node2.y / resolution + dy)
            f.write('  <line x1="%f" y1="%f" x2="%f" y2="%f" style="stroke:rgb(%d,%d,%d)" />\n' % (x0, y0, x1, y1, color[0], color[1], color[2]))
        f.write('</svg>\n')

########################################################################
########################################################################
#
# global values, shared between functions
resolution = None   # microns-per-pixel, from input.json
LINE_WIDTH = 1  # default pen width
DOWNSAMPLE_STEPS = 1    # default image pyramid level
# 
def main(jin):
    global resolution, LINE_WIDTH, DOWNSAMPLE_STEPS
    jout = {}
    spec_id = jin["specimen_id"]

    ####################################################################
    if "line_width" in jin:
        LINE_WIDTH = int(jin["line_width"])

    # according to Staci, accuracy of 20x trace is approximately to the
    #   level of a cell soma (8-10um), which is approx 22 pixels
    # downsampling by 2 won't significantly alter the accuracy of
    #   the annotations and is well within the stated margin of error. this
    #   lends itself to more efficient processing, and better thumbnails
    if "downsample_steps" in jin:
        DOWNSAMPLE_STEPS = int(jin["downsample_steps"])

    ############################################
    # derived constants
    RADS = []
    RADS.append(29)
    RADS.append(15)
    RADS.append(7)
    RADS.append(3)
    DOWNSAMPLE = 1
    for i in range(DOWNSAMPLE_STEPS):
        DOWNSAMPLE *= 2
    GAUS_RAD = RADS[DOWNSAMPLE_STEPS]

    ####################################################################

    # calculate soma position and store in jin structure
    soma_res = jin["soma"]["path"]
    soma_path = soma_res[0].split(',')
    soma_x = np.array(soma_path[0::2], dtype=float)
    soma_y = np.array(soma_path[1::2], dtype=float)
    soma_path = []
    for i in range(len(soma_x)):
        soma_path.append([soma_x[i],soma_y[i]])
    soma_path = np.array(soma_path, np.int32)
    soma_pos = calculate_centroid(soma_x, soma_y)
    jin["soma"]["position"] = soma_pos

    resolution = jin["resolution"]
    swc_name = jin["storage_directory"] + jin["swc_file"]

    ###########################
    # before doing the heavy work, load external objects to make sure they're
    #   available

    # read morphology
    morph = swc.read_swc(swc_name)

    # read 20x
    fname_20x = jin["20x"]["img_path"]
    image_20x = jpeg_twok.read(fname_20x, reduction_factor=DOWNSAMPLE_STEPS)
    abs_width = image_20x.shape[1]
    abs_height = image_20x.shape[0]
    print("20x image size %dx%d at pyramid level %d" % (abs_width, abs_height, DOWNSAMPLE_STEPS))

    # get soma position in 20x pixel space, at present downsample level
    # resolution converts soma coords in microns to pixels
    #   (resolution is microns / pixel)
    resolution *= DOWNSAMPLE    # divide pixels by X means mult res by same
    print("Image resolution (microns/pixel): %f" % resolution)
    dx = jin["soma"]["position"][0] / DOWNSAMPLE - morph.soma_root().x / resolution
    dy = jin["soma"]["position"][1] / DOWNSAMPLE - morph.soma_root().y / resolution
    dx = int(dx)
    dy = int(dy)

    ##############################
    # no point in processing the entire image, as only a small part 
    #   is relevant
    # select min/max values for x,y of all polygons. restrict analysis 
    #   to there
    min_x = 1e10
    min_y = 1e10
    max_x = 0
    max_y = 0
    layers = jin["layers"]
    for layer in layers:
        path_array = np.array(layer["path"].split(','))
        x = np.array(path_array[0::2], dtype=float)
        y = np.array(path_array[1::2], dtype=float)
        min_x = min(min_x, x.min())
        min_y = min(min_y, y.min())
        max_x = max(max_x, x.max())
        max_y = max(max_y, y.max())

    min_x /= DOWNSAMPLE
    min_y /= DOWNSAMPLE
    max_x /= DOWNSAMPLE
    max_y /= DOWNSAMPLE

    # add a border around polygons to provide context
    BORDER = 200
    BORDER /= DOWNSAMPLE
    TOP = min_y - BORDER
    LEFT = min_x - BORDER
    RIGHT = max_x + BORDER
    BOTTOM = max_y + BORDER

    # make sure border doesn't extend beyond image limits
    TOP = max(TOP, 0)
    LEFT = max(LEFT, 0)
    RIGHT = min(RIGHT, abs_width-1)
    BOTTOM = min(BOTTOM, abs_height-1)
    WIDTH = int(RIGHT - LEFT)
    HEIGHT = int(BOTTOM - TOP)

    # adjust soma location for top and left of visible image area
    dx -= LEFT
    dy -= TOP
    #print "inset soma position", dx, dy


    # make frame for each polygon and blur. blur radious should be
    #   approx the size of largest gap or overlap between polygons. this
    #   is for estimating which polygon each point is a best fit in
    layers = jin["layers"]
    for layer in layers:
        path_array = np.array(layer["path"].split(','))
        x = np.array(path_array[0::2], dtype=float)
        x /= DOWNSAMPLE
        x -= LEFT
        y = np.array(path_array[1::2], dtype=float)
        y /= DOWNSAMPLE
        y -= TOP
        #print layer["label"]
        #print x.min(), y.min()
        #print x.max(), y.max()
        xy = []
        for i in range(len(x)):
            xy.append([x[i],y[i]])
        raw_frame = np.zeros((HEIGHT, WIDTH))
        path = np.array(xy)
        cv2.fillPoly(raw_frame, np.int32([path]), 255)
        frame = cv2.blur(raw_frame, (GAUS_RAD, GAUS_RAD))
        layer["frame"] = frame      # blurred polygon
        layer["raw_frame"] = raw_frame  # raw polygon

    # collapse all polys into single array, with value at each position 
    #   corresponding to the index of the polygon that the pixel falls 
    #   into, or -1 if there's no match
    master = np.zeros((HEIGHT, WIDTH, 3))
    master_idx = np.zeros((HEIGHT, WIDTH), dtype=int)
    master_idx -= 1
    for y in range(HEIGHT):
        for x in range(WIDTH):
            peak = 0
            idx = -1
            for i in range(len(layers)):
                frame = layers[i]["frame"]
                val = frame[y][x]
                if val > peak:
                    peak = val
                    idx = i
            if idx >= 0:
                master[y][x] = color_by_index(idx)
                master_idx[y][x] = idx

    #################################################
    # draw standard morphology on colored layers
    draw_morphology(morph, master, int(dx), int(dy))
    outfile = "layer_%d.png" % spec_id
    print "saving " + outfile
    cv2.imwrite(outfile, master)
    jout["morph_layers"] = outfile

    #################################################
    # draw standard morphology on 20x image
    img = image_20x[TOP:BOTTOM,LEFT:RIGHT]
    print img.shape
    draw_morphology(morph, img, int(dx), int(dy))
    outfile = "blockface_%d.png" % spec_id
    print "saving " + outfile
    cv2.imwrite(outfile, img)
    jout["morph_20x"] = outfile

    #################################################
    # associate SWC nodes with morphology layers
    jout["reconstruction_id"] = jin["reconstruction_id"]
    reconstruction = {}
    errs = 0
    for n in morph.node_list:
        x = dx + int(n.x / resolution)
        y = dy + int(n.y / resolution)
        desc = n.to_dict()
        idx = -1
        try:
            idx = master_idx[y][x]
        except:
            errs += 1
        if idx >= 0:
            desc["label"] = layers[idx]["label"]
            n.layer_num = idx
        else:
            desc["label"] = "unknown"
            n.layer_num = -1
        reconstruction[n.n] = desc
    if errs > 0:
        raise Exception("Unable to map %d nodes to a cortical layer" % errs)
    jout["reconstruction"] = reconstruction

    #################################################
    # draw layer & morphology SVG
    outfile = "outline_%d.svg" % spec_id
    print "saving " + outfile
    jout["outline_svg"] = outfile
    write_svg(outfile, jin, morph)
    
    #################################################
    # draw layer-colored morphology on 20x image
    img = image_20x[TOP:BOTTOM,LEFT:RIGHT]
    draw_morphology(morph, img, int(dx), int(dy), True)
    outfile = "layered_blockface_%d.png" % spec_id
    print "saving " + outfile
    cv2.imwrite(outfile, img)
    jout["colored_morph_20x"] = outfile

    #################################################
    # draw layer-colored morphology on empty polygons
    img = np.zeros((HEIGHT, WIDTH, 3))
    layers = jin["layers"]
    for layer in layers:
        path_array = np.array(layer["path"].split(','))
        x = np.array(path_array[0::2], dtype=float)
        x /= DOWNSAMPLE
        x -= LEFT
        y = np.array(path_array[1::2], dtype=float)
        y /= DOWNSAMPLE
        y -= TOP
        for i in range(1,len(x)):
            cv2.line(img, (int(x[i-1]),int(y[i-1])), (int(x[i]),int(y[i])), (255, 255, 255), 1)
        cv2.line(img, (int(x[-1]),int(y[-1])), (int(x[0]),int(y[0])), (255, 255, 255), 1)
    draw_morphology(morph, img, int(dx), int(dy), True)
    outfile = "outline_%d.png" % spec_id
    print "saving " + outfile
    cv2.imwrite(outfile, img)
    jout["colored_morph_poly"] = outfile

    return jout


# mouse
#ims_id = "489909914"
#spec_id = 488679042

#ims_id = "491762612"
#spec_id = 490387590

# human
#ims_id = "487992082"
#spec_id = 488386504

#ims_id = "488759189"
#spec_id = 488418027

#spec_id = 528015670

if __name__ == "__main__":
    module = PipelineModule()
    jin = module.input_data()   # loads input.json
    # "get" input json
    #jin = prep_json(spec_id)
    #json.write("in_%d.json" % spec_id, jin)
    jout = main(jin)
    module.write_output_data(jout)  # writes output.json
    #json.write("out_%d.json" % spec_id, jout)




import numpy as np
import itertools
import allensdk.internal.model.GLM as GLM
import logging
import statsmodels.api as sm
import matplotlib.pyplot as plt

def ASGLM_pairwise(ks_int, I_stim, voltage, spike_ind, cinit, tauinit, SCL, dt, resting_potential, 
                   SHORT_RUN=False, MAKE_PLOT=False, SHOW_PLOT=False, BLOCK=False):
    '''Calculate the resistance and amplitude of the afterspike currents for 
    Parameters
    ----------
        ks_int: list 
            initial possible k's (k=1/tau, where tau is the time constant of the exponential decay)
        I_stim: list of arrays
            input stimulus traces of sweeps
        voltage: list of arrays
            voltage of cell as a result of I_stim
        spike_ind: list of arrays
            each array contains the index of the spikes
        cinit: float 
            membrane capacitance
        tauinit: float
            time constant of membrane
        SCL: float
            number of indicies that should be cut after a spike
        dt: float 
            size of time step of injected current
        Returns
        '''

    #Initialize post-spike filter parameters (MOST OF THESE ARE HARD-CODED currently)
    nkt = 8000                  # arbitrary length of filter  WANT FILTER TO COVER A LENGTH OF TIME, WANT FILTER TO BE LONGER THAN LONGEST LENGTH ASC 
    DTsim = dt                  #DTsim = dt means filter is nkt*dt = 100 ms long in this case THIS SHOULD INDEED BE THE SAMPLE WIDTH
    neye = 0                    #no of identity basis vectors DELTA
    f = 1e-3/dt                 #pre-factor for getting time units correct for bases

    taus_int=[1000./kk for kk in ks_int] #converting to ms
    taus_filter = [f*j for j in taus_int]     #convert ms time to filter time units 1/(dt*ks)
    ks_list = [(1.0/(i)) for i in taus_filter]        #use peak positions of rcos bumps as time-scales for exponential bases
    ncos = 2                    #no of bases!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    flag_exp = 1                #flag_exp = 1 means use exponential bases, else use raised-cosine bumps 
    vL = resting_potential
    
    # GLM fit with post-spike currents
    tst = 0 #190000#355000    #time-step to start
    npcut = int(SCL)  # no of points to cut after each spike-initiation
    
    # Collect spikes between tst and tend  
    t0_list = []
    for mm in range(len(I_stim)):
        tend = len(I_stim[mm])
        t0=[]
        for jj in range(len(spike_ind[mm])):  
            if spike_ind[mm][jj]>tst and spike_ind[mm][jj]<tend:
                t0.append(spike_ind[mm][jj])
        
        t0_list.append(t0)
    
    #Create a list of pairs of ks 
    ks_pairs = list(itertools.combinations(ks_list,ncos)) 
    ks_pairs_in_SI_units= list(itertools.combinations(ks_int, ncos))   
    if len(ks_pairs)!=10:
        raise Exception('figure subplots will need to be changed as there is a different number than 10 ks_pairs.')
    
    #Initialize list to hold charge dump values and amp-vectors
    fitprs_list = []
    llf_list = []
    R_list = []
    #Iterate over all pairs
    if SHORT_RUN:
        logging.warning("You are not doing all the ks pairs in ASGLM_pairwise")
        ks_pairs=[ks_pairs[0]]

    R_for_all_ks_pairs=[]
    asc_amp_for_all_ks_pairs=[]
    El_for_all_ks_pairs=[] 
    C_for_all_ks_pairs=[]
    llh_for_all_ks_pairs=[]
    for ks_ind, (ks_fit_units, ks_SI_units) in enumerate(zip(ks_pairs, ks_pairs_in_SI_units)):
        print 'ks_fit_units', ks_fit_units
        #Create basis IPSPs
        if MAKE_PLOT:
            plotting_colors=['r', 'b', 'g', 'm', 'c']
            plt.figure(78, figsize=(20,10))
        basis_IPSP_list = []
        for rr in range(len(I_stim)): #loop over repeats
            #find the basis of the entire trace
            basis_IPSP, gg0 = GLM.create_basis_IPSP(neye,ncos,taus_filter,ks_fit_units,DTsim,t0_list[rr],I_stim[rr],nkt,flag_exp,npcut)
            basis_IPSP_list.append(basis_IPSP) 
            #--Plot basis IPSPs between si and se    
            si = t0_list[0][0]-10   #plot start_ind
            se = si+nkt+10  #plot end_ind
            tvec = dt*np.arange(si-tst,se-tst)   #convert time-steps to real time (in sec)        
            if MAKE_PLOT:
                plt.figure(78)
                plt.subplot(5,2, ks_ind+1)
                plt.plot(1e3*tvec,basis_IPSP[si:se,:], lw=2, label=str(rr)) #1e3 plots time on x-axis in ms
                plt.xlabel('time (ms)')
                plt.title("k's "+str(ks_SI_units))
        if MAKE_PLOT:
            plt.annotate('ASGLM (fit asc and R): AScurrent basis',
                         xy=(.4, .985),
                         xycoords='figure fraction',
                         horizontalalignment='left', verticalalignment='top',
                         fontsize=20)
            plt.legend()
            plt.tight_layout()
                
        if SHOW_PLOT:
            plt.show(block=BLOCK)                                      
        
        # cut spikes out of dv, v, i, and b_ipsp and put the different sweeps in lists
        i_all_swps_list = []
        b_ipsp_all_swps_list = []
        v_all_swps_list = []
        dv_all_swps_list = []        
        for ss in range(len(I_stim)): #loop over repeats
            tend = len(I_stim[ss])
            i = I_stim[ss][tst:tend-1]
            b_ipsp = basis_IPSP_list[ss][tst:tend-1]
            
            v = voltage[ss][tst:tend-1]    
            vs = voltage[ss][tst+1:tend]
            dv = (vs-v)/dt                  #derivative of voltage
            
            #delete npcut points after spike from each qty    
            delpts = []
            for kk in range(len(t0_list[ss])):
                delpts.append(range(int(t0_list[ss][kk])-tst,int(t0_list[ss][kk])-tst+npcut))
            
            dv = np.delete(dv,delpts,0)
            v = np.delete(v,delpts,0)
            i = np.delete(i,delpts,0)
            b_ipsp = np.delete(b_ipsp,delpts,0)
            
            v_all_swps_list.append(v)
            dv_all_swps_list.append(dv)
            i_all_swps_list.append(i)
            b_ipsp_all_swps_list.append(b_ipsp)
            
            tvec = dt*np.arange(len(v))
        
        # Compute amplitude each basis AS current using a GLM
        if MAKE_PLOT:
            plt.figure(79, figsize=(20, 12))
            plt.figure(80, figsize=(20, 12))
        R_for_each_sweep=[]
        asc_amp_for_each_sweep=[]
        llh_for_each_sweep=[]
        for kkk, (v_spike_deleted, dv_spike_deleted, i_spike_deleted, b_ipsp_spikes_deleted) in \
            enumerate(zip(v_all_swps_list, dv_all_swps_list, i_all_swps_list, b_ipsp_all_swps_list)):
        
            #--fitting afterspike current amplitudes and resistance
            inp = np.zeros((len(i_spike_deleted),ncos+1))
            inp[:,range(0,ncos)] = (1/cinit)*b_ipsp_spikes_deleted[:,range(ncos)]
            inp[:,ncos] = -(v_spike_deleted-vL)/tauinit
            out = dv_spike_deleted -i_spike_deleted/cinit#+ (v-vL)/tauinit - i/cinit

            try:
                glm_fit = sm.GLM(out,inp,family=sm.families.Gaussian(sm.families.links.identity))
                res = glm_fit.fit()
                fitprs = res.params  #fitprs has [AMP OF ASC, TAU]             

                llh=res.llf
                fit_R=tauinit/(fitprs[ncos]*cinit)
                fit_asc_amp=fitprs[:ncos]
                #Compute and plot post-spike current (essentially multiply basis functions with correct amplitudes from GLM fit)
                ipsc = np.sum(b_ipsp_spikes_deleted[:,0:ncos]*fit_asc_amp,1)  #THIS IS TOTAL POSTSPIKE CURRENT
            except Exception, e:
                logging.warning("fit didn't work: " + str(e))
                llh=np.nan
                fit_R=np.NAN
                fit_asc_amp=np.ones(ncos)*np.NAN
                ipsc=np.ones(len(b_ipsp_spikes_deleted[:,0]))*np.NAN
                
            R_for_each_sweep.append(fit_R)
            asc_amp_for_each_sweep.append(fit_asc_amp)
            llh_for_each_sweep.append(llh)
            
            #Plot a single instance of AS current as function of time (in ms)
            if MAKE_PLOT:
                plt.figure(79)
                plt.subplot(5,2, ks_ind+1)
                plot_inds = np.arange(int(t0_list[0][0])-tst,int(t0_list[0][0])-tst+nkt) #plot just after first spike
                tvec = dt*plot_inds
                plt.plot(tvec,ipsc[plot_inds], lw=2, label='llh='+str(llh))
                plt.xlabel('time (s)')
                plt.ylabel('current (A)')
                plt.title("k's "+str(ks_SI_units))
                
                plt.figure(80)
                plt.subplot(5,2,ks_ind+1)
                for ASC, KS in zip(fit_asc_amp, np.array(ks_SI_units)):
                #TODO: MAKE SURE I SHOULD BE USING SI UNITS HERE {I think this is correct as it is what I am returning from the function]
                #TODO: MAKE SURE THE SIGNS OF K ARE OK
                    t_plot=np.arange(10000)*dt
                    single_asc_trace=ASC*np.exp(-KS*t_plot)
                    plt.plot(t_plot, single_asc_trace, lw=2, label='sweep '+str(kkk))
                plt.xlabel('time (s)')
                plt.ylabel('current (A)')
                plt.title("k's "+str(ks_SI_units))

        if MAKE_PLOT:
            plt.figure(79)
            plt.tight_layout()
            plt.annotate('ASGLM (fit asc and R): Sum fit after spike currents',
                         xy=(.3, .985),
                         xycoords='figure fraction',
                         horizontalalignment='left', verticalalignment='top',
                         fontsize=20)
            plt.legend()
            
            plt.figure(80)
            plt.tight_layout()
            plt.annotate('ASGLM (fit asc and R): Individual Currents',
                         xy=(.3, .985),
                         xycoords='figure fraction',
                         horizontalalignment='left', verticalalignment='top',
                         fontsize=20)
            plt.legend()
        if SHOW_PLOT:
            plt.show(block=False)
        
        
#        #BELOW IS OVER EVERY PAIR
        R_for_all_ks_pairs.append(R_for_each_sweep)
        asc_amp_for_all_ks_pairs.append(asc_amp_for_each_sweep)
        llh_for_all_ks_pairs.append(llh_for_each_sweep)

        #!!!!!!!!!!!!!we multiplied ks by dt for SI!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!     
    ave_llh_for_each_pair=np.mean(llh_for_all_ks_pairs, axis=1)
    best_ks_pair_ind = np.where(np.max(ave_llh_for_each_pair)==ave_llh_for_each_pair)[0][0]

    best_k_pair=np.array(ks_pairs[best_ks_pair_ind])/dt
    best_asc_amp=np.array(asc_amp_for_all_ks_pairs[best_ks_pair_ind])
    best_R=np.array(R_for_all_ks_pairs[best_ks_pair_ind])
    best_llh=np.array(llh_for_all_ks_pairs[best_ks_pair_ind])

    print '**********from ASGLM_pairwise******************************************'
    print 'best_ks_pair_ind', best_ks_pair_ind
    print 'best_asc_amp', best_asc_amp
    print 'best_k_pair', best_k_pair
    print 'best_R', best_R 
    print 'best_llh', best_llh
        
    print '**********done with ASGLM_pairwise***********************************'
    
    return best_k_pair, best_asc_amp, best_R, best_llh



# in place of global from neuron import h

def get_h():   
    if get_h.h == None:
        from neuron import h
        get_h.h = h
    return get_h.h
    
get_h.h = None

import sys, os
from cStringIO import StringIO
from output_grabber import OutputGrabber

def load_morphology(filename):
    h = get_h()
    swc = h.Import3d_SWC_read()
    swc.input(str(filename))
    imprt = h.Import3d_GUI(swc, 0)
    h("objref this")
    imprt.instantiate(h.this)


def parse_neuron_output(output_str):
    printed_fields = {}

    for line in output_str.split('\n'):
        if line.startswith('nquad'):
            continue
        toks = line.split()
        if len(toks) == 2:
            v = toks[1].strip()
            try:
                v = float(v)
            except:
                pass
            
            printed_fields[toks[0].strip()] = v

    return printed_fields


def read_neuron_fit_stdout(func):
    def call(*args, **kwargs):

        g = OutputGrabber()
        g.start()
        data = func(*args, **kwargs)
        g.stop()

        printed_fields = parse_neuron_output(g.capturedtext)
        data.update(printed_fields)

        return data

    return call





import multiprocessing as mp
import logging

from six import iteritems
import SimpleITK as sitk
import numpy as np

from base_subimage import run_subimage
from classic_subimage import ClassicSubImage
import image_utilities as iu
from downsampling_utilities import block_average, window_average


#==============================================================================


class ImageSeriesGridder(object):

    @property
    def transform(self):

        if not hasattr(self, '_transform'):
            dfmfld = sitk.ReadImage(str(self.dfmfld_path))
            self._transform = iu.build_composite_transform(dfmfld, self.affine_params)
            del dfmfld
  
        return self._transform


    def __init__(self, in_dims, in_spacing, 
                       out_dims, out_spacing,
                       reduce_level,  
                       subimages, 
                       subimage_kwargs, 
                       nprocesses, 
                       affine_params, 
                       dfmfld_path):
        
        self.in_dims = np.array(in_dims)
        self.in_spacing = np.array(in_spacing)
        
        self.out_dims = np.array(out_dims)
        self.out_spacing = np.array(out_spacing)
        
        self.reduce_level = reduce_level
    
        self.nprocesses = nprocesses
        
        self.affine_params = affine_params
        self.dfmfld_path = dfmfld_path
        
        self.volumes = {}
        
        self.subimages = subimages
        self.subimage_kwargs = subimage_kwargs
        
        
    def set_coarse_grid_parameters(self):
        
        self.coarse_dims, self.coarse_spacing, self.coarse_grid_radius = \
            iu.compute_coarse_parameters(self.in_dims, self.in_spacing, 
                                         self.out_spacing[::-1], 
                                         self.reduce_level)
                                         
        self.coarse_dims[-1] = self.in_dims[-1]
        self.coarse_spacing[-1] = self.in_spacing[-1]
        self.coarse_grid_radius = self.coarse_grid_radius[0]
        
        
    def setup_subimages(self):
        
        if not hasattr(self, 'coarse_grid_radius'):
            self.set_coarse_grid_parameters()
        
        dc = {'in_dims': self.in_dims[:2], 
              'in_spacing': self.in_spacing[:2], 
              'coarse_dims': self.coarse_dims[:2], 
              'coarse_spacing': self.coarse_spacing[:2], 
              'reduce_level': self.reduce_level}
        dc.update(self.subimage_kwargs)
        
        for si in self.subimages:
            si.update(dc)
        
    
    def initialize_coarse_volume(self, key, dtype):
        logging.info('initializing {0} coarse grid volume'.format(key))
        self.volumes[key] = iu.new_image(self.coarse_dims, self.coarse_spacing, dtype, True)

        origin = list(self.volumes[key].GetOrigin())
        origin[2] = 0
        self.volumes[key].SetOrigin(origin)
        
        
    def paste_slice(self, key, index, slice_array):
        '''
        '''
        
        if not key in self.volumes:
            sitk_type = iu.np_sitk_convert(slice_array.dtype)
            self.initialize_coarse_volume(key, sitk_type)
        
        logging.info('resampling data from index {0} into {1} coarse grid volume'.format(index, key))
        slice_image = iu.image_from_array(slice_array.T, self.coarse_spacing[:2], True)
        self.volumes[key] = iu.resample_into_volume(slice_image, None, index, self.volumes[key])
        
    
    def paste_subimage(self, index, output):
        '''Inserts planar accumulators into coarse grid volumes
        '''
        
        for key, array in iteritems(output):
            self.paste_slice(key, index, array)
            output[key] = None
            
        del output
    
    
    def build_coarse_grids(self):
    
        pool = mp.Pool(processes=self.nprocesses)
        mapper = pool.imap_unordered(run_subimage, self.subimages)
        
        logging.info('building coarse grids')
        for index, output in mapper:
            
            logging.info('received coarse planar data from subimage at index {0}'.format(index))
            self.paste_subimage(index, output)
        

    def resample_volume(self, key):
        logging.info('resampling {0} volume'.format(key))
        self.volumes[key] = iu.resample_volume(self.volumes[key], self.out_dims, 
                                               self.out_spacing, None, 
                                               self.transform)


    def consume_volume(self, key, cb):
        logging.info('consuming {0} volume'.format(key))
        self.resample_volume(key)
        cb(self.volumes[key])
        del self.volumes[key]


    def accumulator_to_numpy(self, key, cb):
        self.resample_volume(key)
        cb(self.volumes[key])
        logging.info('converting {0} volume to ndarray'.format(key))
        self.volumes[key] = sitk.GetArrayFromImage(self.volumes[key])


    def make_ratio_volume(self, num_key, den_key, ratio_key):
        '''assume parents numpified
        '''

        self.volumes[ratio_key] = np.divide(self.volumes[num_key], self.volumes[den_key])
        self.volumes[ratio_key][np.isnan(self.volumes[ratio_key])] = 0




import matplotlib
matplotlib.use('agg')

import os, shutil
import allensdk.core.json_utilities as ju
import shutil
import numpy as np
import argparse
import scipy.misc
from scipy.stats import gaussian_kde

import multiprocessing
import functools
import traceback
import logging

from allensdk.brain_observatory.drifting_gratings import DriftingGratings
from allensdk.brain_observatory.static_gratings import StaticGratings
from allensdk.brain_observatory.locally_sparse_noise import LocallySparseNoise
from allensdk.brain_observatory.natural_scenes import NaturalScenes
from allensdk.brain_observatory.natural_movie import NaturalMovie
from allensdk.brain_observatory import observatory_plots as oplots
from allensdk.core.brain_observatory_nwb_data_set import (BrainObservatoryNwbDataSet,
                                                          MissingStimulusException,
                                                          NoEyeTrackingException)
from allensdk.config.manifest import Manifest
from allensdk.internal.core.lims_pipeline_module import PipelineModule, run_module
import allensdk.internal.core.lims_utilities as lu
import allensdk.brain_observatory.stimulus_info as si
from contextlib import contextmanager

LARGE_HEIGHT = 500
SMALL_HEIGHT = 150
SMALL_FONT = 4
LARGE_FONT = 12
PLOT_CONFIGS = { 'small': dict(height_px=SMALL_HEIGHT, pattern="%s_small.png", font_size=SMALL_FONT),
                 'large': dict(height_px=LARGE_HEIGHT, pattern="%s_large.png", font_size=LARGE_FONT),
                 'svg': dict(height_px=LARGE_HEIGHT, pattern="%s.svg", font_size=SMALL_FONT) }
PLOT_TYPES = ["dg", "sg", "ns", "lsn_on", 
              "lsn_off", "rf",
              "nm1", "nm2", "nm3", "sp", 
              "corr", "eye"]

def get_experiment_analysis_file(experiment_id):
    res = lu.query("""
select * from well_known_files wkf 
join well_known_file_types wkft on wkft.id = wkf.well_known_file_type_id
where attachable_id = %d
and wkft.name = 'OphysExperimentCellRoiMetricsFile'
""" % experiment_id)
    return os.path.join(res[0]['storage_directory'], res[0]['filename'])

def get_experiment_nwb_file(experiment_id):
    res = lu.query("""
select * from well_known_files wkf 
join well_known_file_types wkft on wkft.id = wkf.well_known_file_type_id
where attachable_id = %d
and wkft.name = 'NWBOphys'
""" % experiment_id)
    return os.path.join(res[0]['storage_directory'], res[0]['filename'])

def get_experiment_files(experiment_id):
    nwb_file = get_experiment_nwb_file(experiment_id)
    try:
        analysis_file = get_experiment_analysis_file(experiment_id)
    except:
        analysis_file = None

    if not os.path.exists(nwb_file):
        raise Exception("nwb file does not exist: %s" % nwb_file)

    #if not os.path.exists(analysis_file):
   #     raise Exception("analysis file does not exist: %s" % analysis_file)

    return nwb_file, analysis_file

def get_input_data(experiment_id):
    OUTPUT_DIR = "/data/informatics/CAM/analysis/"

    nwb_file, analysis_file = get_experiment_files(experiment_id)
    output_directory = os.path.join(OUTPUT_DIR, str(experiment_id), "thumbnails")
    
    my_file = "/data/informatics/CAM/analysis/%d/%d_analysis.h5" % (experiment_id, experiment_id)

    if os.path.exists(my_file):
        analysis_file = my_file

    input_data = {
        'nwb_file': nwb_file,
        #'analysis_file': analysis_file,
        'analysis_file': analysis_file,
        'output_directory': output_directory
        }

    return input_data

def debug(experiment_id, plots=None, local=False):
    SDK_PATH = "/data/informatics/CAM/analysis/allensdk/"

    input_data = get_input_data(experiment_id)

    run_module(os.path.abspath(__file__),
               input_data,
               input_data["output_directory"],
               sdk_path=SDK_PATH,
               pbs=dict(vmem=32,
                        job_name="bobthumbs_%d"% experiment_id,
                        walltime="10:00:00"),
               local=local,
               optional_args=['--types='+','.join(plots)] if plots else None)

def build_plots(prefix, aspect, configs, output_dir, axes=None, transparent=False):
    Manifest.safe_mkdir(output_dir)

    for config in configs:
        h = config['height_px']
        w = int(h * aspect)
        
        file_name = os.path.join(output_dir, config["pattern"] % prefix)

        logging.debug("file: %s", file_name)
        with oplots.figure_in_px(w, h, file_name, transparent=transparent) as fig:
            matplotlib.rcParams.update({'font.size': config['font_size']})
            yield file_name

def build_cell_plots(cell_specimen_ids, prefix, aspect, configs, output_dir, axes=None, transparent=False):
    for i,csid in enumerate(cell_specimen_ids):
        if np.isnan(csid):
            cell_dir = os.path.join(output_dir, str(i))
        else:
            cell_dir = os.path.join(output_dir, str(csid))

        for fn in build_plots(prefix, aspect, configs, cell_dir, transparent=transparent):
            yield fn, csid, i

def build_drifting_gratings(dga, configs, output_dir):
    for fn in build_plots("drifting_gratings_axes_pref_dir", 1.0, [configs['large'], configs['svg']], output_dir):
        dga.plot_preferred_direction(include_labels=True)
        oplots.finalize_no_axes()

    for fn in build_plots("drifting_gratings_pref_dir", 1.0, [configs['small']], output_dir):
        dga.plot_preferred_direction(include_labels=False)
        oplots.finalize_no_axes()

    for fn in build_plots("drifting_gratings_axes_pref_tf", 1.0, [configs['large'], configs['svg']], output_dir):
        dga.plot_preferred_temporal_frequency()
        oplots.finalize_with_axes()

    for fn in build_plots("drifting_gratings_pref_tf", 1.0, [configs['small']], output_dir):
        dga.plot_preferred_temporal_frequency()
        oplots.finalize_no_labels()

    for fn in build_plots("drifting_gratings_axes_dsi", 1.0, [configs['large'], configs['svg']], output_dir):
        dga.plot_direction_selectivity()
        oplots.finalize_with_axes()

    for fn in build_plots("drifting_gratings_dsi", 1.0, [configs['small']], output_dir):
        dga.plot_direction_selectivity()
        oplots.finalize_no_labels()

    for fn in build_plots("drifting_gratings_axes_osi", 1.0, [configs['large'], configs['svg']], output_dir):
        dga.plot_orientation_selectivity()
        oplots.finalize_with_axes()

    for fn in build_plots("drifting_gratings_osi", 1.0, [configs['small']], output_dir):
        dga.plot_orientation_selectivity()
        oplots.finalize_no_labels()

    csids = dga.data_set.get_cell_specimen_ids()
    for fn, csid, i in build_cell_plots(csids, "drifting_gratings", 1.0, configs.values(), output_dir):
        dga.open_star_plot(csid, include_labels=False, cell_index=i)
        oplots.finalize_no_axes()

def build_static_gratings(sga, configs, output_dir):
    for fn in build_plots("static_gratings_axes_time_to_peak", 1.0, [configs['large'], configs['svg']], output_dir):
        sga.plot_time_to_peak()
        oplots.finalize_with_axes()

    for fn in build_plots("static_gratings_time_to_peak", 1.0, [configs['small']], output_dir):
        sga.plot_time_to_peak()
        oplots.finalize_no_labels()

    for fn in build_plots("static_gratings_axes_pref_ori", 1.5, [configs['large'], configs['svg']], output_dir):
        sga.plot_preferred_orientation(include_labels=True)
        oplots.finalize_no_axes()

    for fn in build_plots("static_gratings_pref_ori", 1.5, [configs['small']], output_dir):
        sga.plot_preferred_orientation(include_labels=False)
        oplots.finalize_no_axes()

    for fn in build_plots("static_gratings_axes_osi", 1.0, [configs['large'], configs['svg']], output_dir):
        sga.plot_orientation_selectivity()
        oplots.finalize_with_axes()

    for fn in build_plots("static_gratings_osi", 1.0, [configs['small']], output_dir):
        sga.plot_orientation_selectivity()
        oplots.finalize_no_labels()

    for fn in build_plots("static_gratings_axes_pref_sf", 1.0, [configs['large'], configs['svg']], output_dir):
        sga.plot_preferred_spatial_frequency()
        oplots.finalize_with_axes()

    for fn in build_plots("static_gratings_pref_sf", 1.0, [configs['small']], output_dir):
        sga.plot_preferred_spatial_frequency()
        oplots.finalize_no_labels()

    csids = sga.data_set.get_cell_specimen_ids()
    for file_name, csid, i in build_cell_plots(csids, "static_gratings_all", 2.0, configs.values(), output_dir):
        sga.open_fan_plot(csid, include_labels=False, cell_index=i)
        oplots.finalize_no_axes()

def build_natural_movie(nma, configs, output_dir, name):
    csids = nma.data_set.get_cell_specimen_ids()
    for file_name, csid, i  in build_cell_plots(csids, name, 1.0, configs.values(), output_dir):
        nma.open_track_plot(csid, cell_index=i)
        oplots.finalize_no_axes()

def build_natural_scenes(nsa, configs, output_dir):
    for fn in build_plots("natural_scenes_axes_time_to_peak", 1.0, [configs['large'], configs['svg']], output_dir):
        nsa.plot_time_to_peak()
        oplots.finalize_with_axes()

    for fn in build_plots("natural_scenes_time_to_peak", 1.0, [configs['small']], output_dir):
        nsa.plot_time_to_peak()
        oplots.finalize_no_labels()

    csids = nsa.data_set.get_cell_specimen_ids()
    for file_name, csid, i in build_cell_plots(csids, "natural_scenes", 1.0, configs.values(), output_dir):
        nsa.open_corona_plot(csid, cell_index=i)
        oplots.finalize_no_axes()

def build_locally_sparse_noise(lsna, configs, output_dir, on):
    prefix = "locally_sparse_noise_" + ("on" if on else "off")

    csids = lsna.data_set.get_cell_specimen_ids()
    for file_name, csid, i in build_cell_plots(csids, prefix, 1.754, [configs['large'], configs['small']], output_dir):
        lsna.open_pincushion_plot(on, cell_specimen_id=csid, cell_index=i)
        oplots.finalize_no_axes()

def build_receptive_field(lsna, configs, output_dir):
    lsn_movie, lsn_mask = lsna.data_set.get_locally_sparse_noise_stimulus_template(lsna.stimulus, 
                                                                                   mask_off_screen=False)

    if lsna.cell_index_receptive_field_analysis_data is None:
        logging.warning("receptive field analysis not performed, so no receptive field plots will be made")
        return

    clim = np.nanpercentile(lsna.receptive_field, [1.0,99.0], axis=None)

    for fn in build_plots("population_receptive_field", 1.754, [configs["large"]], output_dir, transparent=True):
        lsna.plot_population_receptive_field(mask=lsn_mask, scalebar=True)
        oplots.finalize_no_axes()

    for fn in build_plots("population_receptive_field", 1.754, [configs["small"]], output_dir, transparent=True):
        lsna.plot_population_receptive_field(mask=lsn_mask, scalebar=False)
        oplots.finalize_no_axes()

    csids = lsna.data_set.get_cell_specimen_ids()
    for file_name, csid, i in build_cell_plots(csids, "receptive_field_on", 1.754, [configs["large"]], output_dir, transparent=True):
        lsna.plot_cell_receptive_field(True, cell_specimen_id=csid, clim=clim, mask=lsn_mask, cell_index=i, scalebar=True)
        oplots.finalize_no_axes()

    for file_name, csid, i in build_cell_plots(csids, "receptive_field_on", 1.754, [configs["small"]], output_dir, transparent=True):
        lsna.plot_cell_receptive_field(True, cell_specimen_id=csid, clim=clim, mask=lsn_mask, cell_index=i, scalebar=False)
        oplots.finalize_no_axes()

    for file_name, csid, i in build_cell_plots(csids, "receptive_field_off", 1.754, [configs["large"]], output_dir, transparent=True):
        lsna.plot_cell_receptive_field(False, cell_specimen_id=csid, clim=clim, mask=lsn_mask, cell_index=i, scalebar=True)
        oplots.finalize_no_axes()

    for file_name, csid, i in build_cell_plots(csids, "receptive_field_off", 1.754, [configs["small"]], output_dir, transparent=True):
        lsna.plot_cell_receptive_field(False, cell_specimen_id=csid, clim=clim, mask=lsn_mask, cell_index=i, scalebar=False)
        oplots.finalize_no_axes()
       

def build_speed_tuning(analysis, configs, output_dir):
    csids = analysis.data_set.get_cell_specimen_ids()

    for fn in build_plots("running_speed", 1.0, [configs['large'], configs['svg']], output_dir):
        analysis.plot_running_speed_histogram()
        oplots.finalize_with_axes()

    for fn in build_plots("running_speed", 1.0, [configs['small']], output_dir):
        analysis.plot_running_speed_histogram()
        oplots.finalize_no_labels()

    for fn, csid, i in build_cell_plots(csids, "speed_tuning", 1.0, [configs['large'], configs['svg']], output_dir):
        analysis.plot_speed_tuning(csid, cell_index=i)
        oplots.finalize_with_axes()

    for fn, csid, i in build_cell_plots(csids, "speed_tuning", 1.0, [configs['small']], output_dir):
        analysis.plot_speed_tuning(csid, cell_index=i)
        oplots.finalize_no_axes()

def build_correlation_plots(data_set, analysis_file, configs, output_dir):
    sig_corrs = []
    noise_corrs = []

    avail_stims = si.stimuli_in_session(data_set.get_session_type())
    ans = []
    labels = []
    colors = []
    if si.DRIFTING_GRATINGS in avail_stims:
        dg = DriftingGratings.from_analysis_file(data_set, analysis_file)

        if hasattr(dg, 'representational_similarity'):
            ans.append(dg)
            labels.append(si.DRIFTING_GRATINGS_SHORT)
            colors.append(si.DRIFTING_GRATINGS_COLOR)
            setups = [ ( [configs['large']], True ), ( [configs['small']], False )]
            for cfgs, show_labels in setups:
                for fn in build_plots("drifting_gratings_representational_similarity", 1.0, cfgs, output_dir):
                    oplots.plot_representational_similarity(dg.representational_similarity,
                                                            dims=[dg.orivals, dg.tfvals[1:]],
                                                            dim_labels=["dir", "tf"],
                                                            dim_order=[1,0],
                                                            colors=['r','b'],
                                                            labels=show_labels)

    if si.STATIC_GRATINGS in avail_stims:
        sg = StaticGratings.from_analysis_file(data_set, analysis_file)
        if hasattr(sg, 'representational_similarity'):
            ans.append(sg)
            labels.append(si.STATIC_GRATINGS_SHORT)
            colors.append(si.STATIC_GRATINGS_COLOR)
            setups = [ ( [configs['large']], True ), ( [configs['small']], False )]
            for cfgs, show_labels in setups:
                for fn in build_plots("static_gratings_representational_similarity", 1.0, cfgs, output_dir):
                    oplots.plot_representational_similarity(sg.representational_similarity,
                                                            dims=[sg.orivals, sg.sfvals[1:], sg.phasevals],
                                                            dim_labels=["ori", "sf", "ph"],
                                                            dim_order=[1,0,2],
                                                            colors=['r','g','b'], 
                                                            labels=show_labels)

    if si.NATURAL_SCENES in avail_stims:
        ns = NaturalScenes.from_analysis_file(data_set, analysis_file)
        if hasattr(ns, 'representational_similarity'):
            ans.append(ns)
            labels.append(si.NATURAL_SCENES_SHORT)
            colors.append(si.NATURAL_SCENES_COLOR)
            setups = [ ( [configs['large']], True ), ( [configs['small']], False )]
            for cfgs, show_labels in setups:
                for fn in build_plots("natural_scenes_representational_similarity", 1.0, cfgs, output_dir):
                    oplots.plot_representational_similarity(ns.representational_similarity, labels=show_labels)

    if len(ans):
        for an in ans:
            sig_corrs.append(an.signal_correlation)
            extra_dims = range(2,len(an.noise_correlation.shape))
            noise_corrs.append(an.noise_correlation.mean(axis=tuple(extra_dims)))

        for fn in build_plots("correlation", 1.0, [configs['large'], configs['svg']], output_dir):
            oplots.population_correlation_scatter(sig_corrs, noise_corrs, labels, colors, scale=16.0)
            oplots.finalize_with_axes()

        for fn in build_plots("correlation", 1.0, [configs['small']], output_dir):
            oplots.population_correlation_scatter(sig_corrs, noise_corrs, labels, colors, scale=4.0)
            oplots.finalize_no_labels()

        csids = ans[0].data_set.get_cell_specimen_ids()
        for fn, csid, i in build_cell_plots(csids, "signal_correlation", 1.0, [configs['large']], output_dir):
            row = ans[0].row_from_cell_id(csid, i)
            oplots.plot_cell_correlation([ np.delete(sig_corr[row],i) for sig_corr in sig_corrs ], 
                                         labels, colors)
            oplots.finalize_with_axes()

        for fn, csid, i in build_cell_plots(csids, "signal_correlation", 1.0, [configs['small']], output_dir):
            row = ans[0].row_from_cell_id(csid, i)
            oplots.plot_cell_correlation([ np.delete(sig_corr[row],i) for sig_corr in sig_corrs ], 
                                         labels, colors)
            oplots.finalize_no_labels()

        
def lsna_check_hvas(data_set, data_file):
    avail_stims = si.stimuli_in_session(data_set.get_session_type())    
    targeted_structure = data_set.get_metadata()['targeted_structure']

    stim = None

    if targeted_structure == "VISp":
        if si.LOCALLY_SPARSE_NOISE_4DEG in avail_stims:
            stim = si.LOCALLY_SPARSE_NOISE_4DEG
        elif si.LOCALLY_SPARSE_NOISE in avail_stims:
            stim = si.LOCALLY_SPARSE_NOISE
    else:
        if si.LOCALLY_SPARSE_NOISE_8DEG in avail_stims:
            stim = si.LOCALLY_SPARSE_NOISE_8DEG
        elif si.LOCALLY_SPARSE_NOISE in avail_stims:
            stim = si.LOCALLY_SPARSE_NOISE

    if stim is None:
        raise MissingStimulusException("Could not find appropriate LSN stimulus for session %s", 
                                       data_set.get_session_type())
    else:
        logging.debug("in structure %s, using %s stimulus for plots", targeted_structure, stim)
            
    
    return LocallySparseNoise.from_analysis_file(data_set, data_file, stim)


def build_eye_tracking_plots(data_set, configs, output_dir):
    try:
        pupil_times, xy_deg = data_set.get_pupil_location()
        xy_deg = xy_deg[~np.isnan(xy_deg).any(axis=1)]
        c = gaussian_kde(xy_deg.T)(xy_deg.T)

        for fn in build_plots("eye_tracking_gaze_axes", 1.0,
                              [configs['large'], configs['svg']],
                              output_dir):
            oplots.plot_pupil_location(xy_deg, c=c, include_labels=True)
            oplots.finalize_with_axes()

        for fn in build_plots("eye_tracking_gaze", 1.0, [configs['small']],
                              output_dir):
            oplots.plot_pupil_location(xy_deg, c=c, include_labels=False)
            oplots.finalize_no_axes()
    except NoEyeTrackingException:
        logging.debug("No eye tracking found.")
        

def build_type(nwb_file, data_file, configs, output_dir, type_name):
    data_set = BrainObservatoryNwbDataSet(nwb_file)
    try:
        if type_name == "dg":
            dga = DriftingGratings.from_analysis_file(data_set, data_file)
            build_drifting_gratings(dga, configs, output_dir)
        elif type_name == "sg":
            sga = StaticGratings.from_analysis_file(data_set, data_file)
            build_static_gratings(sga, configs, output_dir)
        elif type_name == "nm1":
            nma = NaturalMovie.from_analysis_file(data_set, data_file, si.NATURAL_MOVIE_ONE)
            build_natural_movie(nma, configs, output_dir, si.NATURAL_MOVIE_ONE)
        elif type_name == "nm2":
            nma = NaturalMovie.from_analysis_file(data_set, data_file, si.NATURAL_MOVIE_TWO)
            build_natural_movie(nma, configs, output_dir, si.NATURAL_MOVIE_TWO)
        elif type_name == "nm3":
            nma = NaturalMovie.from_analysis_file(data_set, data_file, si.NATURAL_MOVIE_THREE)
            build_natural_movie(nma, configs, output_dir, si.NATURAL_MOVIE_THREE)
        elif type_name == "ns":
            nsa = NaturalScenes.from_analysis_file(data_set, data_file)
            build_natural_scenes(nsa, configs, output_dir)
        elif type_name == "sp":
            nma = NaturalMovie.from_analysis_file(data_set, data_file, si.NATURAL_MOVIE_ONE)
            build_speed_tuning(nma, configs, output_dir)
        elif type_name == "lsn_on":
            lsna = lsna_check_hvas(data_set, data_file)
            build_locally_sparse_noise(lsna, configs, output_dir, True)
        elif type_name == "lsn_off":
            lsna = lsna_check_hvas(data_set, data_file)
            build_locally_sparse_noise(lsna, configs, output_dir, False)
        elif type_name == "rf":
            lsna = lsna_check_hvas(data_set, data_file)
            build_receptive_field(lsna, configs, output_dir)
        elif type_name == "corr":
            build_correlation_plots(data_set, data_file, configs, output_dir)
        elif type_name == "eye":
            build_eye_tracking_plots(data_set, configs, output_dir)

    except MissingStimulusException as e:
        logging.warning("could not load stimulus (%s)", type_name)
    except Exception as e:
        traceback.print_exc()
        logging.critical("error running stimulus (%s)", type_name)
        raise e

def parse_input(data):
    nwb_file = data.get("nwb_file", None)

    if nwb_file is None:
        raise IOError("input JSON missing required field 'nwb_file'")
    if not os.path.exists(nwb_file):
        raise IOError("nwb file does not exists: %s" % nwb_file)

    analysis_file = data.get("analysis_file", None)

    if analysis_file is None:
        raise IOError("input JSON missing required field 'analysis_file'")
    if not os.path.exists(analysis_file):
        raise IOError("analysis file does not exists: %s" % analysis_file)


    output_directory = data.get("output_directory", None)

    if output_directory is None:
        raise IOError("input JSON missing required field 'output_directory'")

    Manifest.safe_mkdir(output_directory)
    
    return nwb_file, analysis_file, output_directory
    
def build_experiment_thumbnails(nwb_file, analysis_file, output_directory, 
                                types=None, threads=4):
    if types is None:
        types = PLOT_TYPES

    logging.info("nwb file: %s", nwb_file)
    logging.info("analysis file: %s", analysis_file)
    logging.info("output directory: %s", output_directory)
    logging.info("types: %s", str(types))
    Manifest.safe_mkdir(output_directory)

    if len(types) == 1:
        build_type(nwb_file, analysis_file, PLOT_CONFIGS, output_directory, types[0])
    elif threads == 1:
        for type_name in types:
            build_type(nwb_file, analysis_file, PLOT_CONFIGS, output_directory, type_name)
    else:
        p = multiprocessing.Pool(threads)

        func = functools.partial(build_type, nwb_file, analysis_file, PLOT_CONFIGS, output_directory)
        results = p.map(func, types)
        p.close()
        p.join()

    

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-t", "--threads", type=int, default=4)
    parser.add_argument("--log-level", default=logging.DEBUG)
    parser.add_argument("--types", default=','.join(PLOT_TYPES))
    parser.add_argument("input_json")
    parser.add_argument("output_json")
    args = parser.parse_args()

    args.types = args.types.split(',')

    logging.getLogger().setLevel(args.log_level)

    input_data = ju.read(args.input_json)

    nwb_file, analysis_file, output_directory = parse_input(input_data)

    build_experiment_thumbnails(nwb_file, analysis_file, output_directory, 
                                args.types, args.threads)

    ju.write(args.output_json, {})

if __name__ == "__main__": main()



import logging

#import SimpleITK as sitk
import nrrd
import numpy as np

#def read(path):
#    return np.swapaxes(sitk.GetArrayFromImage(sitk.ReadImage(str(path))), 0, 2)    

def read(path):
    return np.ascontiguousarray(nrrd.read(path)[0])



def load_annotation(annotation_path, data_mask_path=None):
    '''Read data files segmenting the reference space into regions of valid 
    and invalid data, then further among brain structures
    '''
    
    logging.info('getting annotation')
    annotation = read(annotation_path)
    
    logging.info('casting to signed')
    #  It shouldn't matter now, but there may be future structures with ids 
    #  sufficiently large that we need that extra bit
    logging.debug('max annotated value: {0}'.format(np.amax(annotation)))
    annotation = annotation.astype(np.int32)
    logging.debug('max annotated value: {0}'.format(np.amax(annotation)))
    
    logging.info('negating left hemisphere')
    logging.debug('min annotated value: {0}'.format(np.amin(annotation)))
    lr_mid = int( np.round(annotation.shape[2] / 2) )
    annotation[:, :, :lr_mid] = annotation[:, :, :lr_mid] * -1
    logging.debug('min annotated value: {0}'.format(np.amin(annotation)))

    if data_mask_path is not None:
        logging.info('getting_data_mask')
        data_mask =read(data_mask_path)
        
        logging.info('applying data mask')
        annotation[np.logical_not(data_mask)] = 0
   
    return annotation


def get_sum_pixels(sum_pixels_path):    
    logging.info('getting sum_pixels')
    return {'sum_pixels': read(sum_pixels_path)}
    

def get_sum_pixel_intensities(sum_pixel_intensities_path, injection_sum_pixel_intensities_path):
    logging.info('getting sum pixel intensities')
    return {'sum_pixel_intensities': read(sum_pixel_intensities_path),
            'injection_sum_pixel_intensities': read(injection_sum_pixel_intensities_path)}


def get_cav_density(cav_density_path):    
    logging.info('getting cav density')
    return {'cav_density': read(cav_density_path)}


def get_injection_data(injection_fraction_path, injection_density_path, 
                       injection_energy_path):
    '''Read nrrd files containing injection signal data
    '''

    logging.info('getting injection_fraction')
    injection_fraction = read(injection_fraction_path)
    
    logging.info('getting injection_sum_projecting_pixels')
    injection_density =  read(injection_density_path)
    
    logging.info('getting injection_energy')
    injection_energy = read(injection_energy_path)
    
    return {'injection_fraction': injection_fraction, 
            'injection_density': injection_density, 
            'injection_energy': injection_energy}
            
            
def get_projection_data(projection_density_path, projection_energy_path, 
                        aav_exclusion_fraction_path=None):
    '''Read nrrd files containing global signal data
    '''
    
    logging.info('getting projection density')
    projection_density =  read(projection_density_path)
    
    logging.info('getting projection energy')
    projection_energy = read(projection_energy_path)
    
    try:
        logging.info('getting aav exclusion fraction')
        aav_exclusion_fraction = read(aav_exclusion_fraction_path)
        aav_exclusion_fraction[aav_exclusion_fraction > 0] = 1
        aav_exclusion_fraction = aav_exclusion_fraction.astype(np.bool_, order='C')
    
    except (IOError, OSError, RuntimeError):
        logging.info('skipping aav exclusion fraction')
        aav_exclusion_fraction = np.zeros(projection_density.shape, dtype=np.bool_, order='C')
    
    return {'projection_density': projection_density, 
            'projection_energy': projection_energy, 
            'aav_exclusion_fraction': aav_exclusion_fraction}



"""Run annotated region metrics calculations"""
import logging
import os
import h5py
from allensdk.internal.core.lims_utilities import get_input_json
from allensdk.internal.brain_observatory.annotated_region_metrics import get_metrics
from allensdk.internal.core.lims_pipeline_module import (PipelineModule,
                                                         run_module)

SDK_PATH = "/data/informatics/CAM/isi_metrics/allensdk"
SCRIPT_PATH = ("/data/informatics/CAM/isi_metrics/allensdk/allensdk/internal"
               "/pipeline_modules/run_annotated_region_metrics.py")

def debug(region_id, storage_directory="./", local=True,
          sdk_path=SDK_PATH, script_path=SCRIPT_PATH, lims_host="lims2"):
    strategy_class = "AnnotatedRegionMetricsStrategy"
    object_class = "AnnotatedRegion"
    input_json = get_input_json(region_id, object_class, strategy_class,
                                lims_host)
    exp_dir = os.path.join(storage_directory, str(region_id))
    run_module(script_path,
               input_json,
               exp_dir,
               sdk_path=sdk_path,
               pbs=dict(vmem=4,
                        job_name="isi_metrics_{}".format(region_id),
                        walltime="1:00:00"),
                        local=local)


def load_arrays(h5_file):
    with h5py.File(h5_file, "r") as f:
        altitude_phase = f['retinotopy_altitude'][:]
        azimuth_phase = f['retinotopy_azimuth'][:]
    return altitude_phase, azimuth_phase


def main():
    mod = PipelineModule()
    data = mod.input_data()

    h5_file = data["processed_h5"]
    altitude_phase, azimuth_phase = load_arrays(h5_file)
    del data["processed_h5"]

    output_data = get_metrics(altitude_phase, azimuth_phase, **data)

    mod.write_output_data(output_data)

if __name__ == "__main__": main()



#!/usr/bin/python
import os, sys
import allensdk.internal.core.swc as swc


def resave_swc(orig_swc, new_file):
    """ Reads SWC file into AllenSDK Morphology object and resaves
    it. This can fix some problems in an SWC file that may disrupt
    other software tools reading the file (e.g., NEURON)

    Parameters
    ----------
        orig_swc: string
        Name of SWC file to read

        new_file: string
        Name of output SWC file
    """
    try:
        morphology = swc.read_swc(orig_swc)
    except:
        print("Failed to read SWC file '%s'" % orig_swc)
        raise
    try:
        morphology.save(new_file)
    except:
        print("Failed to save SWC file '%s'" % new_file)


class TestNode(object):
    def __init__(self, n, t, x, y, z, r, pn):
        # these values correspond to columns in an SWC file
        self.n = n
        self.t = t
        self.x = x
        self.y = y
        self.z = z
        self.r = r
        self.pn = pn
        self.children = []  # IDs of child nodes

    def __str__(self):
        """ create string with node information in succinct, 
        single-line form """
        return "%d %d %.4f %.4f %.4f %.4f %d %s" % (self.n, self.t, self.x, self.y, self.z, self.r, self.pn, str(self.children))



def validate_swc(swc_file):
    """ 
    Tests SWC files for compatibility with AllenSDK

    To be compatible with NEURON, SWC files must have the following properties:
        1) a single root node with parent ID '-1'
        2) sequentially increasing ID numbers
        3) immediate children of the soma cannot branch

    To be compatible with feature analysis, SWC files can only have node
    types in the range 1-4:
        1 = soma
        2 = axon
        3 = [basal] dendrite
        4 = apical dendrite
    """
    success = True  # be optimistic

    # see if SWC file is readable by internal tools
    print("Validating " + swc_file)
    try:
        morphology = swc.read_swc(swc_file)
    except:
        print("Fatal error reading SWC file")
        return False

    for node in morphology.node_list:
        if node.t < 1 or node.t > 4:
            print("Expecting type between 1 and 4, but found %d" % node.t)
            print("File has unrecognized node type(s)")
            print("----------------------------------")
            success = False
            break

    # make sure all dendrite nodes are in tree 0
    # this is because modeling requires a full dendrite morphology
    for node in morphology.node_list:
        if (node.t == 3 or node.t == 4) and node.tree_id != 0:
            print("Dendrite node(s) exist in disconnected trees")
            print("This breaks an SDK modeling requirement")
            print("----------------------------------")
            success = False
            break

    # if we've made it here, file is OK for using Morphology class, and 
    #   should be valid with internal processing. It may also be able
    #   to be convertable for NEURON use by resaving it

    nodes = []
    node_table = [] # lookup table by node num
    line_num = 1
    try:
        with open(swc_file, "r") as f:
            for line in f:
                # remove comments
                if line.lstrip().startswith('#'):
                    continue
                # read values. expected SWC format is:
                #   ID, type, x, y, z, rad, parent
                # x, y, z and rad are floats. the others are ints
                toks = line.split()
                vals = TestNode(
                        n =  int(toks[0]),
                        t =  int(toks[1]),
                        x =  float(toks[2]),
                        y =  float(toks[3]),
                        z =  float(toks[4]),
                        r =  float(toks[5]),
                        pn = int(toks[6].rstrip()),
                    )
                # store this node
                while len(nodes) <= vals.n:
                    nodes.append(None)
                nodes[vals.n] = vals
                #nodes.append(vals)
                #
                if vals.n < 0:
                    print("Negative node ID not allowed")
                    print("Node: " + str(vals))
                    return False
                while vals.n >= len(node_table):
                    node_table.append(None)
                node_table[vals.n] = vals
                # increment line number (used for error reporting only)
                line_num += 1
    except:
        err = "File not recognized as valid SWC file.\n"
        err += "Problem parsing line %d\n" % line_num
        if line is not None:
            err += "Content: '%s'\n" % line
        raise IOError(err)

    try:
        for node in nodes:
            if node is None:
                continue
            par = None
            if node.pn >= 0:
                par = node_table[node.pn]
                par.children.append(node.n)
    except:
        print("Error reading SWC file -- fail to link child to parent")
        print("Node:    %s" % str(node))
        print("----------------------------------")
        success = False

    # verify presence and number of soma and root nodes
    num_soma_nodes = sum([ int(c is not None and c.t == 1) for c in nodes ])
    if num_soma_nodes == 0:
        print("SWC must have at least one soma node.  Found: %d" % num_soma_nodes)
        print("----------------------------------")
        success = False
    elif num_soma_nodes > 1:
        print("Warning: File has multiple soma nodes. This can interfere with feature analysis in some external software (e.g., vaa3d)")
        print("----------------------------------")

    num_root_nodes = sum([ int(c is not None and c.pn == -1) for c in nodes ])
    # case of no root nodes covered by rule below that ID of child must
    #   be greater than that of parent
    if num_root_nodes > 1:
        print("Warning: File has multiple root nodes. This can interfere with feature analysis in some external software (e.g., vaa3d)")
        print("----------------------------------")

    # get a list of all of the ids, make sure they are unique while we're at it
    all_ids = set()
    for node in nodes:
        if node is None:
            continue
        iid = int(node.n)
        if iid in all_ids:
            print("Node ID %s is not unique." % node.n)
            print("----------------------------------")
            success = False
            break
        pid = int(node.pn)
        if iid < pid:
            print("Node (%d) has a smaller ID that its parent (%d)" % (iid, pid))
            print("----------------------------------")
            success = False
            break
        all_ids.add(iid)
        
    # make sure that first root node is soma
    for n in nodes:
        if n is not None:
            root = n
            break
    #root = nodes[0]
    if root.t != 1:
        # see if soma has a root
        if sum([int(c is not None and c.t == 2 and c.pn == -1) for c in nodes]) == 0:
            print("No soma root found in file")
            print("----------------------------------")
            success = False
        print("First root node is not soma")
        print("This should be fixable by calling resave_swc() on the file if there is a soma root in the file")
        print("----------------------------------")
        success = False

    # verify that children of the root have max one child
    for root_child_id in root.children:
        root_child = nodes[root_child_id]
        num_grand_children = len(root_child.children)
        if num_grand_children > 1:
            print("Child of root (%s) has more than one child (%d)" % ( root_child_id, num_grand_children ))
            print("----------------------------------")
            success = False

    # sort the ids and make sure there are no gaps
    sorted_ids = sorted(all_ids)
    for i in xrange(1, len(sorted_ids)):
        if sorted_ids[i] - sorted_ids[i-1] != 1:
            print("Node IDs are not sequential")
            print("This can be fixed by calling resave_swc() on the file")
            print("----------------------------------")
            success = False
    return success
    
    
    
def main():
    argc = len(sys.argv)
    if argc < 1:
        print("usage: python %s <swc_file> [<swc_file ...]")
        print("")
        print("Validate an SWC file for use with NEURON")
        sys.exit(1)
    try:
        for i in range(1, argc):
            if validate_swc(sys.argv[i]) == True:
                print "    PASS"
            else:
                print "    FAIL"
                exit(1)
    except Exception as e:
        print "    FAIL"
        print str(e)
        exit(1)
if __name__ == "__main__": main()



import sys
import os
from allensdk.internal.core.lims_pipeline_module import run_module, PipelineModule
import allensdk.internal.core.lims_utilities as lu
import h5py
import allensdk.brain_observatory.roi_masks as roi_masks
import numpy as np
import logging


def create_roi_masks(rois, w, h, motion_border):
    roi_list = []

    for roi in rois:
        mask = np.array(roi["mask"], dtype=bool)
        px = np.argwhere(mask)
        px[:,0] += roi["y"]
        px[:,1] += roi["x"]

        mask = roi_masks.create_roi_mask(w, h, motion_border, 
                                         pix_list=px[:,[1,0]], 
                                         label=str(roi["id"]), 
                                         mask_group=roi.get("mask_page",-1))

        roi_list.append(mask)

    # sort by roi id
    roi_list.sort(key=lambda x: x.label)

    return roi_list

def main():
    mod = PipelineModule()
    jin = mod.input_data()

    # find width and height of movie
    stack_file = jin["motion_corrected_stack"]
    with h5py.File(stack_file, "r") as f:
        d = f["data"]
        h = d.shape[1]
        w = d.shape[2]

    # where to store output
    output_directory = jin["storage_directory"]

    # motion border
    border = [jin["motion_border"]["x0"],
              jin["motion_border"]["x1"],
              jin["motion_border"]["y0"],
              jin["motion_border"]["y1"]]

    # create roi mask objects
    roi_mask_list = create_roi_masks(jin["rois"], w, h, border)
    roi_names = [ roi.label for roi in roi_mask_list ]

    # extract traces
    roi_traces, neuropil_traces = roi_masks.calculate_roi_and_neuropil_traces(jin["motion_corrected_stack"], roi_mask_list, border)

    # write ROI traces
    roi_file = os.path.join(output_directory, "roi_traces.h5")
    logging.debug("Writing " + roi_file)
    rf = h5py.File(roi_file, 'w')
    rf["data"] = roi_traces
    rf.create_dataset("roi_names", data=roi_names)
    rf.close()

    # write neuropil traces
    np_file = os.path.join(output_directory, "neuropil_traces.h5")
    logging.debug("Writing " + np_file)
    neup = h5py.File(np_file, 'w')
    neup["data"] = neuropil_traces
    neup.create_dataset("roi_names", data=roi_names)
    neup.close()

    # output json 
    jout = {}
    jout["neuropil_trace_file"] = np_file
    jout["roi_trace_file"] = roi_file

    mod.write_output_data(jout)
   

def get_input_data(experiment_id, output_dir):
    
    sd = lu.query("select storage_directory from ophys_experiments where id = %d" % experiment_id)[0]['storage_directory']
    rois = lu.query("select * from cell_rois where ophys_experiment_id = %d" % experiment_id)

    motion_border = None
    nrois = []
    for roi in rois:
        if motion_border is None:
            motion_border=dict(y1=roi['max_correction_up'],
                               y0=roi['max_correction_down'],
                               x0=roi['max_correction_left'],
                               x1=roi['max_correction_right'])
        nrois.append({
                'width': roi['width'],
                'height': roi['height'], 
                'x': roi['x'],
                'y': roi['y'],
                'id': roi['id'],
                'valid': roi['valid_roi'],
                'mask': roi['mask_matrix']
                })

    movie_path = os.path.join(sd, "processed", "concat_31Hz_0.h5")
    with h5py.File(movie_path, "r") as f:
        s = f["data"].shape
        movie_height = s[1]
        movie_width = s[2]
    

    return dict(
        image=dict(width=movie_width, height=movie_height),
        motion_border=motion_border,
        motion_corrected_stack=movie_path,
        storage_directory=output_dir,
        rois=nrois
        )

def debug(experiment_id, local=False):
    OUTPUT_DIRECTORY = "/data/informatics/CAM/traces"
    PYTHON = "/shared/utils.x86_64/python-2.7/bin/python"
    SDK_PATH = "/data/informatics/CAM/traces/allensdk" 
    SCRIPT = "/data/informatics/CAM/traces/allensdk/allensdk/internal/pipeline_modules/run_ophys_trace_extraction.py"

    exp_dir = os.path.join(OUTPUT_DIRECTORY, str(experiment_id))
    input_data = get_input_data(experiment_id, exp_dir)

    run_module(SCRIPT, 
               input_data, 
               exp_dir, 
               sdk_path=SDK_PATH,
               pbs=dict(vmem=16,
                        job_name="extraces_%d"% experiment_id,
                        walltime="10:00:00"),
               local=local)

if __name__ == "__main__": main()
    
    



# Copyright 2016-2017 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.


import json
import os
import logging
import allensdk.internal.core.lims_utilities as lims_utilities
from allensdk.config.manifest_builder import ManifestBuilder
from allensdk.config.manifest import Manifest


class BiophysicalModuleReader(object):
    STIMULUS_CONTENT_TYPE = None
    MORPHOLOGY_TYPE_ID = 303941301
    MOD_FILE_TYPE_ID = 292178729

    def __init__(self):
        self._log = logging.getLogger(__name__)
        self.lims_path = None
        self.lims_data = None
        self.lims_update_data = None

    def read_lims_message(self, message, lims_path):
        self.lims_path = lims_path
        self.lims_data = message[0]
        self.lims_update_data = dict(self.lims_data)

    def read_lims_file(self, lims_path):
        self.lims_path = lims_path
        self.read_json(lims_path)
        self.lims_update_data = dict(self.lims_data)

    def read_json(self, path):
        with open(path, 'rb') as f:
            self.read_json_string(f.read())

    def read_json_string(self, json_string):
        self.lims_data = json.loads(json_string)
        self.lims_update_data = dict(self.lims_data)

    def stimulus_file_entries(self):
        ''' read the well known file path from the lims result
            corresponding to the stimulus file
            :return: well_known_file entries
            :rtype: array of dicts
        '''
        neuronal_model = self.lims_data['neuronal_model']
        specimen = neuronal_model['specimen']
        roi_result = specimen['ephys_roi_result']
        well_known_files = roi_result['well_known_files']

        stimulus_file_entries = []

        for well_known_file in well_known_files:
            try:
                file_type_id = well_known_file['well_known_file_type_id']
                
                if file_type_id == lims_utilities.NWB_FILE_TYPE_ID:
                    stimulus_file_entries.append(well_known_file)
            except:
                self._log.warn('skipping well known file record with no well known file type.')

        return stimulus_file_entries

    def stimulus_path(self):
        ''' Get the path to the stimulus file from the lims result.
            :return: path to stimulus file
            :rtype: string
        '''
        file_entries = self.stimulus_file_entries()

        if len(file_entries) > 1:
            self._log.warning('More than one stimulus file found.')

        file_entry = file_entries[0]

        stimulus_path = os.path.join(file_entry['storage_directory'],
                                     file_entry['filename'])

        return stimulus_path

    def lims_working_directory(self):
        ''' While this is the same directory as the neuronal_model_run
            directory, it can be mocked out for testing if the
            other directory is read only.
        '''
        return self.neuronal_model_run_dir()

    def neuronal_model_run_dir(self):
        ''' read the directory path where
            output goes from the lims optimization config json
            
            Parameters
            ----------
            
            Returns
            -------
            string:
                directory path
        '''
        return self.lims_data['storage_directory']

    def fit_parameters_file_entries(self):
        ''' read the fit_parameter file path from the lims result
            corresponding to the stimulus file
            :return: well_known_file entries
            :rtype: array of dicts
        '''
        neuronal_model = self.lims_data['neuronal_model']
        well_known_files = neuronal_model['well_known_files']

        fit_parameter_file_entries = []

        for well_known_file in well_known_files:
            file_type_id = well_known_file['well_known_file_type_id']

            if file_type_id == lims_utilities.MODEL_PARAMETERS_FILE_TYPE_ID:
                fit_parameter_file_entries.append(well_known_file)

        return fit_parameter_file_entries

    def fit_parameters_path(self):
        ''' Get the path to the fit parameters file from the lims result.
            :return: path to file
            :rtype: string
        '''
        file_entries = self.fit_parameters_file_entries()

        if len(file_entries) > 1:
            self._log.warning('More than one fit parameter file found.')

        file_entry = file_entries[0]

        fit_parameter_path = os.path.join(file_entry['storage_directory'],
                                          file_entry['filename'])

        return fit_parameter_path

    def model_type(self):
        ''' TODO: comment
        '''
        return self.lims_data['neuronal_model']['neuronal_model_template']['name']

    def morphology_file_entries(self):
        ''' read the well known file paths
            from the lims result corresponding to the morphology
            
            Returns
            -------
            arrary of dicts:
                well known file entries
        '''
        neuronal_model = self.lims_data['neuronal_model']
        specimen = neuronal_model['specimen']
        reconstructions = specimen['neuron_reconstructions']

        morphology_file_entries = []

        for reconstruction in reconstructions:
            superseded = reconstruction['superseded']
            manual = reconstruction['manual']

            if manual == True and superseded == False:
                well_known_files = reconstruction['well_known_files']

                for well_known_file in well_known_files:
                    file_type_id = well_known_file['well_known_file_type_id']
                    
                    if file_type_id == BiophysicalModuleReader.MORPHOLOGY_TYPE_ID:
                        morphology_file_entries.append(well_known_file)

        return morphology_file_entries

    def morphology_path(self):
        ''' Get the path to the morphology file from the lims result.
            :return: path to morphology file
            :rtype: string
        '''
        file_entries = self.morphology_file_entries()

        if len(file_entries) > 1:
            self._log.warning('More than one morphology file found.')

        file_entry = file_entries[0]

        morphology_path = os.path.join(file_entry['storage_directory'],
                                     file_entry['filename'])

        return morphology_path

    def sweep_entries(self):
        ''' read the sweep entries
            from the lims result corresponding to the stimulus
            :return: stimulus sweep entries
            :rtype: array of dicts
        '''
        neuronal_model = self.lims_data['neuronal_model']
        specimen = neuronal_model['specimen']
        sweeps = specimen['ephys_sweeps']

        return sweeps

    def sweep_numbers_by_type(self):
        sweeps = self.sweep_entries()

        d = {s['ephys_stimulus']['ephys_stimulus_type']['name']: [] for s in sweeps}

        for n, s in enumerate(sweeps):
            t = s['ephys_stimulus']['ephys_stimulus_type']['name']
            d[t].append(s['sweep_number'])

        return d

    def sweep_numbers(self):
        ''' Get the stimulus sweep numbers from the lims result
            :return: list of sweep numbers
            :rtype: array of ints
        '''
        sweep_entries = self.sweep_entries()

        if not sweep_entries or len(sweep_entries) < 1:
            self._log.warning('No sweeps found.')

        sweeps = [sweep_entry['sweep_number'] \
                  for sweep_entry in sweep_entries \
                  if sweep_entry['workflow_state'] == 'auto_passed' or \
                     sweep_entry['workflow_state'] == 'manual_passed' ]

        return list(set(sweeps))

    def mod_file_entries(self):
        ''' read the NERUON .mod file entries
            from the lims result corresponding to the NeuronModel
            :return: well known file entries
            :rtype: array of dicts
        '''
        neuronal_model = self.lims_data['neuronal_model']
        model_template = neuronal_model['neuronal_model_template']
        well_known_files = model_template['well_known_files']

        mod_file_entries = []

        for well_known_file in well_known_files:
            file_type_id = well_known_file['well_known_file_type_id']
                            
            if file_type_id == BiophysicalModuleReader.MOD_FILE_TYPE_ID:
                mod_file_entries.append(well_known_file)
                
        return mod_file_entries

    def mod_file_paths(self):
        ''' Get the paths to the mod files from the lims result.
            :return: paths to mod files
            :rtype: array of strings
        '''
        file_entries = self.mod_file_entries()

        if not file_entries or len(file_entries) < 1:
            self._log.warning('No mod files found.')

        mod_file_paths = []

        for file_entry in file_entries:
            mod_path = os.path.join(file_entry['storage_directory'],
                                    file_entry['filename'])
            self._log.info(mod_path)
            mod_file_paths.append(mod_path)

        return mod_file_paths

    def update_well_known_file(self,
                               path,
                               well_known_file_type_id=None):
        if well_known_file_type_id == None:
            well_known_file_type_id = \
                lims_utilities.NWB_UNCOMPRESSED_FILE_TYPE_ID
        well_known_files = self.lims_data['well_known_files']
        (dirname, filename) = os.path.split(os.path.abspath(path))

        def get_nwb_file_id(f):
            if ('well_known_file_type_id' in f and
                f['well_known_file_type_id'] == well_known_file_type_id and
                os.path.normpath(f['storage_directory']) == 
                os.path.normpath(dirname) and
                f['filename'] == filename):
                return f['id']
            else:
                return None

        def not_nwb_file(f):
            if ('well_known_file_type_id' in f and
                f['well_known_file_type_id'] == well_known_file_type_id):
                return False
            else:
                return True

        try:
            existing_file_id = \
                next(wkf_id for wkf_id
                     in (get_nwb_file_id(f2)
                         for f2 in well_known_files)
                     if wkf_id)

            # existing parameter file found
            self.lims_update_data['well_known_files'] = \
                [f for f in well_known_files if not_nwb_file(f)]

            self.lims_update_data['well_known_files'] += [{
                    'id': existing_file_id,
                    'content_type': None,
                    'filename': filename,
                    'storage_directory': dirname,
                    'well_known_file_type_id': well_known_file_type_id
                }]
        except StopIteration:
            # no matching nwb files found, remove possible unmatching
            self.lims_update_data['well_known_files'] = \
                [f for f in well_known_files if not_nwb_file(f)]
            
            (dirname, filename) = os.path.split(os.path.abspath(path))
            self.lims_update_data['well_known_files'] += [{
                    'content_type': None,
                    'filename': filename,
                    'storage_directory': dirname,
                    'well_known_file_type_id': well_known_file_type_id
                }]

    def set_workflow_state(self, state):
        self.lims_update_data['workflow_state'] = state

    def write_file(self, path):
        with open(path, 'wb') as f:
            f.write(json.dumps(self.lims_update_data, indent=2))

    def to_manifest(self, manifest_path=None):
        b = ManifestBuilder()
        b.add_path('BASEDIR', os.path.realpath(os.curdir))

        b.add_path('WORKDIR', 
                   os.path.realpath(os.curdir))

        b.add_path('MORPHOLOGY',
                   self.morphology_path(),
                   typename='file')
        b.add_path('CODE_DIR', 'templates')
        b.add_path('MODFILE_DIR', 'modfiles')

        for modfile in self.mod_file_entries():
            b.add_path('MOD_FILE_%s' % (os.path.splitext(modfile['filename'])[0]),
                       os.path.join(modfile['storage_directory'],
                                    modfile['filename']),
                       typename='file',
                       format='MODFILE')

        b.add_path('neuronal_model_run_data',
                   self.lims_path,
                   typename='file')

        b.add_path('stimulus_path',
                   self.stimulus_path(),
                   typename='file',
                   format='NWB')

        b.add_path('manifest',
                   os.path.join(os.path.realpath(os.curdir),
                                manifest_path),
                   typename='file')

        neuronal_model_run_id = self.lims_data['id']

        nwb_file_name, extension = \
            os.path.splitext(os.path.basename(self.stimulus_path()))
        b.add_path('output_path',
                   '%d_virtual_experiment%s' % (neuronal_model_run_id,
                                                extension),
                   typename='file',
                   parent_key='WORKDIR',
                   format='NWB')

        b.add_path('fit_parameters',
                   self.fit_parameters_path())
        
        b.add_section('biophys',
                      {"biophys": [{"model_file": [ manifest_path,
                                    self.fit_parameters_path() ],
                                    "model_type": self.model_type()}]})

        b.add_section('stimulus_conf',
                      {"runs": [{"neuronal_model_run_id": 
                                     neuronal_model_run_id,
                                 "sweeps": self.sweep_numbers(),
                                 "sweeps_by_type": self.sweep_numbers_by_type()
                                 }]})

        b.add_section('hoc_conf',
                      {"neuron" : [{"hoc": ["stdgui.hoc",
                                            "import3d.hoc",
                                            "cell.hoc" ]
                                    }]}) 

        m = Manifest(config=b.path_info)

        if manifest_path != None:
            b.write_json_file(manifest_path, overwrite=True)

        return m



import numpy as np

def are_two_lists_of_arrays_the_same(data1, data2):   
    '''returns False if to lists of arrays are different.
    otherwise the function returns True.
    '''
    
    if len(data1) != len(data2): 
        return False
    for a,b in zip(data1,data2):
        if np.any(a != b):
            return False
    
    return True
        
        


import os, platform, re, logging
from allensdk.core.json_utilities import read_url_get

HDF5_FILE_TYPE_ID = 306905526
NWB_FILE_TYPE_ID = 475137571
NWB_UNCOMPRESSED_FILE_TYPE_ID = 478840678
NWB_DOWNLOAD_FILE_TYPE_ID = 481007198
METHOD_CONFIG_FILE_TYPE_ID = 324440685
MODEL_PARAMETERS_FILE_TYPE_ID = 329230374
BIOPHYS_MODEL_PARAMETERS_FILE_TYPE_ID = 329230374


def get_well_known_files_by_type(wkfs, wkf_type_id):
    out = [ os.path.join( wkf['storage_directory'], wkf['filename'] )
            for wkf in wkfs
            if wkf.get('well_known_file_type_id',None) == wkf_type_id ]

    if len(out) == 0:
        raise IOError("Could not find well known files with type %d." % wkf_type_id)
    
    return out


def get_well_known_file_by_type(wkfs, wkf_type_id):
    out = get_well_known_files_by_type(wkfs, wkf_type_id)

    nout = len(out)
    if nout != 1:
        raise IOError("Expected single well known file with type %d. Got %d." % (wkf_type_id, nout))
    
    return out[0]


def get_well_known_files_by_name(wkfs, filename):
    out = [ os.path.join( wkf['storage_directory'], wkf['filename'] )
            for wkf in wkfs
            if wkf['filename'] == filename ]

    if len(out) == 0:
        raise IOError("Could not find well known files with name %s." % filename)

    return out

def get_well_known_file_by_name(wkfs, filename):
    out = get_well_known_files_by_name(wkfs, filename)

    nout = len(out)
    if nout != 1:
        raise IOError("Expected single well known file with name %s. Got %d." % (filename, nout))
    
    return out[0]

def append_well_known_file(wkfs, path, wkf_type_id=None, content_type=None):
    record = {
        'filename': os.path.basename(path),
        'storage_directory': os.path.dirname(path)
        }

    if wkf_type_id is not None:
        record['well_known_file_type_id'] = wkf_type_id

    if content_type is not None:
        record['content_type'] = content_type

    for wkf in wkfs:
        if wkf['filename'] == record['filename']:
            logging.debug("found existing well known file record for %s, updating", path)
            wkf.update(record)
            return

    logging.debug("could not find existing well known file record for %s, appending", path)
    wkfs.append(record)

def _connect(user="limsreader", host="limsdb2", database="lims2", password="limsro", port=5432):
    import pg8000

    conn = pg8000.connect(user=user, host=host, database=database, password=password, port=port)
    return conn, conn.cursor()

def _select(cursor, query):
    cursor.execute(query)
    columns = [ d[0] for d in cursor.description ]
    return [ dict(zip(columns, c)) for c in cursor.fetchall() ]

def select(cursor, query):
    raise DeprecationWarning("lims_utilities.select is deprecated.  Please use lims_utilities.query instead.")

def connect(user="limsreader", host="limsdb2", database="lims2", password="limsro", port=5432):
    raise DeprecationWarning("lims_utilities.connect is deprecated.  Please use lims_utilities.query instead.")

def query(query, user="limsreader", host="limsdb2", database="lims2", password="limsro", port=5432):
    conn, cursor = _connect(user, host, database, password, port)
    try:
        results = _select(cursor, query)
    finally:
        cursor.close()
        conn.close()
    return results

def safe_system_path(file_name):
    if platform.system() == "Windows":
        return linux_to_windows(file_name)
    else:
        return convert_from_titan_linux(os.path.normpath(file_name))

def convert_from_titan_linux(file_name):
    # Lookup table mapping project to program
    project_to_program= {
        "neuralcoding": "braintv", 
        '0378': "celltypes",
        'conn': "celltypes",
        'ctyconn': "celltypes",
        'humancelltypes': "celltypes",
        'mousecelltypes': "celltypes",
        'shotconn': "celltypes",
        'synapticphys': "celltypes",
        'whbi': "celltypes",
        'wijem': "celltypes"
    }
    # Tough intermediary state where we have old paths
    # being translated to new paths
    m = re.match('/projects/([^/]+)/vol1/(.*)', file_name)
    if m:
        newpath = os.path.normpath(os.path.join(
            '/allen',
            'programs',
            project_to_program.get(m.group(1),'undefined'),
            'production',
            m.group(1),
            m.group(2)
        ))
        return newpath
    return file_name

def linux_to_windows(file_name):
    # Lookup table mapping project to program
    project_to_program= {
        "neuralcoding": "braintv", 
        '0378': "celltypes",
        'conn': "celltypes",
        'ctyconn': "celltypes",
        'humancelltypes': "celltypes",
        'mousecelltypes': "celltypes",
        'shotconn': "celltypes",
        'synapticphys': "celltypes",
        'whbi': "celltypes",
        'wijem': "celltypes"
    }

    # Simple case for new world order
    m = re.match('/allen', file_name)
    if m:
        return "\\" + file_name.replace('/','\\')

    # /data/ paths are being retained (for now)
    # this will need to be extended to map directories to
    # /allen/{programs,aibs}/workgroups/foo
    m = re.match('/data/([^/]+)/(.*)', file_name)
    if m:
        return os.path.normpath(os.path.join('\\\\aibsdata', m.group(1), m.group(2)))

    # Tough intermediary state where we have old paths
    # being translated to new paths
    m = re.match('/projects/([^/]+)/vol1/(.*)', file_name)
    if m:
        newpath = os.path.normpath(os.path.join(
            '\\\\allen',
            'programs',
            project_to_program.get(m.group(1),'undefined'),
            'production',
            m.group(1),
            m.group(2)
        ))
        return newpath

    # No matches found.  Clean up and return path given to us
    return os.path.normpath(file_name)


def get_input_json(object_id, object_class, strategy_class, host="lims2",
                   **kwargs):
    query_string = ("http://{}/InputJsons?strategy_class={}"
                    "&object_class={}&object_id={}").format(host,
                                                            strategy_class,
                                                            object_class,
                                                            object_id)
    for key, value in kwargs.items():
        query_string += "&{}={}".format(key, value)

    return read_url_get(query_string)



import os.path
import numpy as np
import json, logging
from allensdk.core.nwb_data_set import NwbDataSet
import allensdk.core.json_utilities as ju
from allensdk.model.biophys_sim.config import Config
from allensdk.internal.model.biophysical import ephys_utils
from allensdk.internal.model.biophysical.deap_utils import Utils

class Report:
    _log = logging.getLogger('allensdk.model.biophysical.make_deap_fit_json')

    def __init__(self,
                 top_level_description,
                 fit_type):
        self.utils = None
        self.top_level_description = top_level_description
        self.description = None
        self.manifest = None
        self.specimen_id = str(self.top_level_description.data['runs'][0]['specimen_id'])
        self.fit_type = fit_type
        self.target_path = self.top_level_description.manifest.get_path('target_path')
        self.target = ju.read(self.target_path)
    
        self.seeds = [1234, 1001, 4321, 1024, 2048]
        self.org_selections = [0, 100, 500, 1000] # Picks thek best, 100th best, etc. organisms as examples
        self.trace_colors = ["#1b9e77", "#d95f02", "#7570b3", "#e7298a"]
        
        self.config_path = self.top_level_description.manifest.get_path('fit_config_json',
                                                                        self.fit_type)
        self.fit_config = Config().load(self.config_path)
        
        fit_style_path = self.fit_config.data["biophys"][0]["model_file"][-1]
        
        self.fit_style_info = ju.read(fit_style_path)

        self.used_features = self.fit_style_info["features"]
        self.all_params = self.fit_style_info["channels"] + self.fit_style_info["addl_params"]
        
        nwb_path = self.top_level_description.manifest.get_path('stimulus_path')        
        self.data_set = NwbDataSet(nwb_path)
        
        self.neuronal_model_data = ju.read(self.top_level_description.manifest.get_path('neuronal_model_data'))
        self.specimen_data = self.neuronal_model_data['specimen']
        self.all_sweeps = self.specimen_data['ephys_sweeps']
        
    
    
    def best_fit_value(self):
        return self.all_hof_fit_errors[self.sorted_indexes[self.org_selections[0]]]
    
    
    def generate_fit_file(self):
        self.gather_from_seeds()
        self.setup_model()
        self.check_org_selections_for_noise_block()
        self.make_fit_json_file()
    
    
    def make_fit_json_file(self):
        json_data = {}
        
        # passive
        json_data["passive"] = [{}]
        json_data["passive"][0]["ra"] = self.target["passive"][0]["ra"]
        json_data["passive"][0]["e_pas"] = self.target["passive"][0]["e_pas"]
        json_data["passive"][0]["cm"] = []
        for k in self.target["passive"][0]["cm"]:
            json_data["passive"][0]["cm"].append({"section": k, "cm": self.target["passive"][0]["cm"][k]})
        
        # fitting
        json_data["fitting"] = [{}]
        json_data["fitting"][0]["sweeps"] = self.target["fitting"][0]["sweeps"]
        json_data["fitting"][0]["junction_potential"] = self.target["fitting"][0]["junction_potential"]
        
        # conditions
        json_data["conditions"] = self.fit_style_info["conditions"]
        json_data["conditions"][0]["v_init"] = self.target["passive"][0]["e_pas"]
        
        # genome
        json_data["genome"] = []
        genome_vals = self.all_hof_fits[self.sorted_indexes[self.org_selections[0]], :]
        for i, p in enumerate(self.all_params):
            if len(p["mechanism"]) > 0:
                param_name = p["parameter"] + "_" + p["mechanism"]
            else:
                param_name = p["parameter"]
            json_data["genome"].append({"value": genome_vals[i],
                                        "section": p["section"],
                                        "name": param_name,
                                        "mechanism": p["mechanism"]
                                        })
        
        # write out file
        with open(self.top_level_description.manifest.get_path('output_fit_file',
                                                               self.specimen_id,
                                                               self.fit_type), "w") as f:
            json.dump(json_data, f, indent=2)
    
            
    def setup_model(self):
        morphology_path = os.path.realpath(self.top_level_description.manifest.get_path('MORPHOLOGY'))
        cwd = os.path.realpath(os.curdir)
        self.utils = Utils(self.fit_config)
        h = self.utils.h
        self.utils.generate_morphology(morphology_path)
        self.utils.load_cell_parameters()
        self.utils.insert_iclamp()
        self.stim_params = self.fit_config.data["stimulus"][0]
        self.utils.set_iclamp_params(self.stim_params["amplitude"],
                                     self.stim_params["delay"],
                                     self.stim_params["duration"])
        h.tstop = self.stim_params["delay"] * 2.0 + self.stim_params["duration"]
        h.cvode_active(1)
        h.cvode.atolscale("cai", 1e-4)
        h.cvode.maxstep(10)
    
    
    def gather_from_seeds(self):
        first_created = False
        for s in self.seeds:
            final_hof_fit_path = \
                self.top_level_description.manifest.get_path('final_hof_fit',
                                                             self.fit_type,
                                                             s)
            final_hof_path = \
                self.top_level_description.manifest.get_path('final_hof',
                                                             self.fit_type,
                                                             s)
            if not os.path.exists(final_hof_fit_path):
                Report._log.warn("Could not find output file %s for seed %d" % (final_hof_fit_path, s))
                continue
            
            hof_fit_errors = np.loadtxt(final_hof_fit_path)
            hof_fits = np.loadtxt(final_hof_path)
            if not first_created:
                all_hof_fit_errors = hof_fit_errors.copy()
                all_hof_fits = hof_fits.copy()
                first_created = True
            else:
                all_hof_fit_errors = np.hstack([all_hof_fit_errors, hof_fit_errors])
                all_hof_fits = np.vstack([all_hof_fits, hof_fits])
        self.all_hof_fits = all_hof_fits
        self.all_hof_fit_errors = all_hof_fit_errors
        self.sorted_indexes = np.argsort(self.all_hof_fit_errors)

    
    def check_org_selections_for_noise_block(self):
        h = self.utils.h
        v_vec, i_vec, t_vec = self.utils.record_values()

        depol_block_threshold = -50.0 # mV
        block_min_duration = 50.0 # ms

        h.cvode_active(0)
        noise_i_stim = []
        for sweep_type in ["C1NSSEED_1", "C1NSSEED_2"]:
            sweeps, sweep_numbers, statuses = ephys_utils.get_sweeps_of_type(sweep_type, self.all_sweeps)
            _, expt_i, expt_t = ephys_utils.get_sweep_v_i_t_from_set(self.data_set, sweep_numbers[0])
            noise_i_stim.append(expt_i)
        dt = (expt_t[1] - expt_t[0]) * 1e3
        h.dt = dt
        h.tstop = expt_t[-1] * 1e3
        self.utils.stim.dur = 1e12

        for ii, org_ind in enumerate(self.sorted_indexes):
            Report._log.debug("Testing org %s%s" % (ii, org_ind))
            self.utils.set_actual_parameters(self.all_hof_fits[org_ind, :])
            depol_okay = True
            use_ii = -1
            for expt_i in noise_i_stim:
                Report._log.debug("Running some noise")
                i_stim_vec = h.Vector(expt_i * 1e-3)
                i_stim_vec.play(self.utils.stim._ref_amp, dt)
                h.finitialize()
                h.run()
                i_stim_vec.play_remove()

                v = v_vec.as_numpy()
                t = t_vec.as_numpy()
                i = i_vec.as_numpy()
                stim_start_idx = 0
                stim_end_idx = len(t) - 1
                bool_v = np.array(v > depol_block_threshold, dtype=int)
                up_indexes = np.flatnonzero(np.diff(bool_v) == 1)
                down_indexes = np.flatnonzero(np.diff(bool_v) == -1)
                if len(up_indexes) > len(down_indexes):
                    down_indexes = np.append(down_indexes, [stim_end_idx])

                if len(up_indexes) != 0:
                    max_depol_duration = np.max([t[down_indexes[k]] - t[up_idx] for k, up_idx in enumerate(up_indexes)])
                    if max_depol_duration > block_min_duration:
                        Report._log.debug("Encountered depolarization block")
                        depol_okay = False
                        break
            if depol_okay:
                Report._log.debug("Did not detect depolarization block on noise traces")
                use_ii = ii
                break
        h.cvode_active(1)
        self.utils.set_iclamp_params(self.stim_params["amplitude"], self.stim_params["delay"],
            self.stim_params["duration"])
        self.utils.h.tstop = self.stim_params["delay"] * 2.0 + self.stim_params["duration"]

        if use_ii == -1:
            Report._log.debug("Could not find an organism without depolarization block on noise.")
        else:
            self.org_selections = [o + use_ii for o in self.org_selections]



# Copyright 2015-2016 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.

import csv
import copy
import math
from allensdk.internal.morphology.morphology import *
from allensdk.internal.morphology.node import Node

########################################################################
def read_swc(file_name):
    """  
    Read in an SWC file and return a Morphology object.

    Parameters
    ----------
    file_name: string
        SWC file name.

    Returns
    -------
    Morphology
        A Morphology instance.
    """
    nodes = []
    line_num = 1
    try:
        with open(file_name, "r") as f:
            for line in f:
                # remove comments
                if line.lstrip().startswith('#'):
                    continue
                # read values. expected SWC format is:
                #   ID, type, x, y, z, rad, parent
                # x, y, z and rad are floats. the others are ints
                toks = line.split()
                vals = Node(
                        n =  int(toks[0]),
                        t =  int(toks[1]),
                        x =  float(toks[2]),
                        y =  float(toks[3]),
                        z =  float(toks[4]),
                        r =  float(toks[5]),
                        pn = int(toks[6].rstrip())
                    )
                # store this node
                nodes.append(vals)
                # increment line number (used for error reporting only)
                line_num += 1
    except ValueError:
        err = "File not recognized as valid SWC file.\n"
        err += "Problem parsing line %d\n" % line_num
        if line is not None:
            err += "Content: '%s'\n" % line
        raise IOError(err)

    return Morphology(node_list=nodes)    


########################################################################
class Marker( dict ): 
    """ Simple dictionary class for handling reconstruction marker objects. """

    SPACING = [ .1144, .1144, .28 ]

    CUT_DENDRITE = 10 
    NO_RECONSTRUCTION = 20

    def __init__(self, *args, **kwargs):
        super(Marker, self).__init__(*args, **kwargs)

        # marker file x,y,z coordinates are offset by a single image-space pixel
        self['x'] -= self.SPACING[0]
        self['y'] -= self.SPACING[1]
        self['z'] -= self.SPACING[2]
        


def read_marker_file(file_name):
    """ read in a marker file and return a list of dictionaries """

    with open(file_name, 'r') as f:
        rows = csv.DictReader((r for r in f if not r.startswith('#')), 
                              fieldnames=['x','y','z','radius','shape','name','comment',
                                          'color_r','color_g','color_b'])

        return [ Marker({ 'x': float(r['x']), 
                          'y': float(r['y']), 
                          'z': float(r['z']), 
                          'name': int(r['name']) }) for r in rows ]




from __future__ import division
import multiprocessing as mp
import logging
import functools
import os

from six import iteritems
import SimpleITK as sitk
import numpy as np

from base_subimage import run_subimage
from classic_subimage import ClassicSubImage
from image_series_gridder import ImageSeriesGridder
from downsampling_utilities import downsample_average
from image_utilities import image_from_array, write_volume


#==============================================================================

    
def run_classic(grid_prefix, accumulator_prefix, target_spacings, **input_data):

    paths = []

    isg = ImageSeriesGridder(**input_data)
    isg.setup_subimages()

    isg.build_coarse_grids()

    cb = functools.partial(write_volume, name='sum_pixel_intensities', prefix=accumulator_prefix, paths=paths)
    isg.consume_volume('sum_pixel_intensities', cb)

    cb = functools.partial(write_volume, name='injection_sum_pixel_intensities', prefix=accumulator_prefix, paths=paths)
    isg.consume_volume('injection_sum_pixel_intensities', cb)

    cb = functools.partial(write_volume, name='sum_pixels', prefix=accumulator_prefix, paths=paths)
    isg.accumulator_to_numpy('sum_pixels', cb)

    ratio_and_pyramid(isg, 'sum_projecting_pixels',  'sum_pixels', 'projection_density', 
                      accumulator_prefix, grid_prefix, target_spacings, paths=paths)

    ratio_and_pyramid(isg, 'injection_sum_projecting_pixels',  'sum_pixels', 'injection_density', 
                      accumulator_prefix, grid_prefix, target_spacings, paths=paths)

    ratio_and_pyramid(isg, 'sum_projecting_pixel_intensities',  'sum_pixels', 'projection_energy', 
                      accumulator_prefix, grid_prefix, target_spacings, paths=paths)

    ratio_and_pyramid(isg, 'injectionsum_projecting_pixel_intensities',  'sum_pixels', 'injection_energy', 
                      accumulator_prefix, grid_prefix, target_spacings, paths=paths)

    ratio_and_pyramid(isg, 'injection_sum_pixels',  'sum_pixels', 'injection_fraction', 
                      accumulator_prefix, grid_prefix, target_spacings, paths=paths)

    ratio_and_pyramid(isg, 'aav_exclusion_sum_pixels',  'sum_pixels', 'aav_exclusion_fraction', 
                      accumulator_prefix, grid_prefix, target_spacings, paths=paths)

    isg.volumes['data_mask'] = isg.volumes['sum_pixels'] / np.amax(isg.volumes['sum_pixels'])
    del isg.volumes['sum_pixels']
    isg.volumes['data_mask'][isg.volumes['data_mask'] > 0.5] = 1
    isg.volumes['data_mask'][isg.volumes['data_mask'] < 0.5] = 0
    handle_pyramid(isg, 'data_mask', target_spacings, grid_prefix, paths=paths)

    return paths


def handle_pyramid(isg, key, target_spacings, prefix, paths):

    cspacing = isg.out_spacing[0]
    for tspacing in target_spacings:

        downsampled = downsample_average(isg.volumes[key], cspacing, tspacing)
        write_volume(image_from_array(downsampled, [tspacing] * 3), 
                     key, prefix=prefix, specify_resolution=tspacing, paths=paths)

    del isg.volumes[key]


def ratio_and_pyramid(isg, num, den, out, accumulator_prefix, grid_prefix, target_spacings, paths):

    cb = functools.partial(write_volume, name=num, prefix=accumulator_prefix, paths=paths)
    isg.accumulator_to_numpy(num, cb)
        
    isg.make_ratio_volume(num, den, out)
    del isg.volumes[num]
    handle_pyramid(isg, out, target_spacings, grid_prefix, paths)


#==============================================================================








































from __future__ import division
import itertools as it
import logging

from skimage.measure import block_reduce
from skimage.util import view_as_windows
from scipy.ndimage.filters import convolve
import numpy as np


def downsample_average(volume, current_spacing, target_spacing):
    
    factor = target_spacing / current_spacing

    if factor == 1:
        return volume

    if factor - np.floor(factor) == 0:
        volume = block_average(volume, factor)
    elif factor - np.floor(factor) == 0.5:
        volume = window_average(volume, factor)
    else:
        raise ValueError('voxels cannot be unevenly split!')

    return volume


def block_average(volume, factor):
    logging.info('downsampling by block averaging with a factor of {0}'.format(factor))
    factor = np.around(factor).astype(int)
    return block_reduce(volume, tuple([factor, factor, factor]), np.mean, 0)


def apply_divisions(image, window_size):

    for axis in xrange(image.ndim):

        slc = [slice(window_size-1, None, window_size) 
               if ii == axis else slice(0, None) 
               for ii in xrange(image.ndim)]

        image[slc] = image[slc] / 2


def window_average(volume, factor):
    logging.info('downsampling by window averaging with a factor of {0}'.format(factor))
    volume = volume.copy()

    window_size = np.ceil(factor).astype(int)
    window_step = 2 * window_size - 1
    output_size = np.ceil([sh / factor for sh in volume.shape]).astype(int)

    apply_divisions(volume, window_size)
    volume = conv(volume, factor, window_size)

    return extract(volume, factor, window_size, window_step, output_size)


def conv(image, factor, window_size):
    kernel = np.ones([window_size for ii in image.shape])  
    return convolve(image, kernel, mode='constant', cval=0.0) / factor ** image.ndim


def extract(image, factor, window_size, window_step, output_shape):
  
    output = np.zeros( output_shape )

    for case in it.product(*([[0, 1]] * image.ndim)):     

        inp = [slice(window_size - 2, None, window_step) 
               if not ii else slice(window_size, None, window_step) for ii in case]
        out = [slice(0, None, 2) if not ii else slice(1, None, 2) for ii in case]

        output[out] = image[inp]

    return output




from __future__ import division
import logging
import functools
from collections import defaultdict

import numpy as np

from interval_unionizer import IntervalUnionizer
from cav_unionize import CavUnionize


class CavUnionizer(IntervalUnionizer):


    @classmethod
    def record_cb(cls):
        return CavUnionize()

  
    @classmethod
    def propagate_record(cls, child_record, ancestor_record, copy_all=False):
        return child_record.propagate(ancestor_record)


    def extract_data(self, data_arrays, low, high):
        '''As parent
        '''
    
        unionize = self.__class__.record_cb()
        unionize.calculate(low, high, data_arrays)
        
        return unionize 


    def postprocess_unionizes(self, raw_unionizes, image_series_id, volume_scale):

        unionizes = []
        
        logging.info('getting formatted unionize output')
        for sid, un in raw_unionizes.iteritems():
            
            if sid < 0:
                hemisphere = 'left'
            else:
                hemisphere = 'right'
            sid = abs(sid)
                
            out = un.output(volume_scale)
            out['structure_id'] = sid
            out['hemisphere'] = hemisphere
            out['image_series_id'] = image_series_id

            unionizes.append(out)

        return unionizes



from __future__ import division
import logging
import functools
from collections import defaultdict
import copy as cp

import numpy as np
from six import iteritems



class IntervalUnionizer(object):

    
    @classmethod
    def record_cb(cls):
        return defaultdict(lambda *a, **k: 0, {})


    def __init__(self, exclude_structure_ids=None):
        '''Builds unionize records from grid data. Unionize records are 
        summaries of experimental observations occuring in a particular 
        spatial domain. Domains are generally specified by the intersection of 
        
            1. a brain structure
            2. an injection polygon (or its inverse)
            3. the left or right side of the brain
        
        Parameters
        ----------
        exclude_structure_ids : list of int, optional
            Don't generate records for these structures. Defaults to [0], 
            which excludes everything not in the brain.
        
        '''
    
        if exclude_structure_ids is None:
            exclude_structure_ids = [0]
        self.exclude_structure_ids = exclude_structure_ids
    
    
    def setup_interval_map(self, annotation):
        '''Build a map from structure ids to intervals in the sorted flattened 
        reference space. 
        
        Parameters
        ----------
        annotation : np.ndarray
            Segmentation label array.
        
        '''
    
        logging.info('getting flat annotation')
        flat_annot = annotation.flat
        
        logging.info('finding sort')
        self.sort = np.argsort(flat_annot)
        
        logging.info('sorting flat annotation')
        flat_annot = flat_annot[self.sort]
        
        logging.info('finding bounds')
        diff = np.diff(flat_annot)
        bounds = np.nonzero(diff)[0]
        uniques = [ flat_annot[ii] for ii in bounds ] + [flat_annot[-1]]
        
        logging.info('building map')
        lower_bounds = [0] + (bounds + 1).tolist()
        upper_bounds = (bounds + 1).tolist() + [len(flat_annot)]
        self.interval_map = {sid: item for sid, item 
                             in zip(uniques, zip(lower_bounds, upper_bounds)) 
                             if sid not in self.exclude_structure_ids}
        
        
    def extract_data(self, data_arrays, low, high, **kwargs):
        '''Given flattened data arrays and a specified interval, generate 
        summary data
        
        Parameters
        ----------
        data_arrays : dict
            Keys identify types of data volume. Values are flattened, sorted 
            arrays.
        low : int
            Index at which interval of interest begins. Inclusive.
        high : int
            Index at which interval of interest ends. Exclusive.
        
        '''
    
        raise NotImplementedError('specify in subclass!')
        
        
    @classmethod
    def propagate_record(cls, child_record, ancestor_record, copy_all=False):
        '''Updates one unionize corresponding to a rootward structure with 
        information from a unionize corresponding to a leafward structure
        
        Parameters
        ----------
        child_record : unionize
            Data will be drawn from this record
        ancestor_record : unionize
            This record will be updated
        
        '''
    
        raise NotImplementedError('specify in subclass!')
        
        
    @classmethod
    def propagate_unionizes(cls, direct_unionizes, ancestor_id_map):
        '''Structures are arranged in a tree, whose leafward-oriented edges 
        indicate physical containment. This method updates rootward unionize 
        records with information from leafward ones.
        
        Parameters
        ----------
        direct_unionizes : list of unionizes
            Each entry is a unionize record produced from a collection of 
            directly labeled voxels in the segmentation volume.
        ancestor_id_map : dict
            Keys are structure ids. Values are ids of all structures rootward in 
        the tree, including the key node
            
        Returns
        -------
        output_unionizes : list of unionizes
            Contains completed unionize records at all depths in the structure 
            tree
        
        '''

        
        output_unionizes = defaultdict(cls.record_cb, cp.deepcopy(direct_unionizes))
        for k, v in iteritems(direct_unionizes):
            for aid in ancestor_id_map[k]:

                if k == aid:
                    continue
                
                logging.debug('propagating data from {0} to {1}'.format(k, aid))
                output_unionizes[aid] = cls.propagate_record(v, output_unionizes[aid])
                
        return output_unionizes
        

    @classmethod
    def propagate_to_bilateral(cls, lateral_unionizes):

        bilateral = defaultdict(cls.record_cb, {})
        for sid in list(lateral_unionizes.keys()):
            unionize = lateral_unionizes[sid]
            other_id = -1 * sid

            if (sid in bilateral) or (other_id in bilateral):
                continue

            logging.debug('bilateralizing structure {0}'.format(sid))
            other = lateral_unionizes[other_id]
      
            bilateral[sid] = cls.propagate_record(unionize, bilateral[sid], True)
            bilateral[sid] = cls.propagate_record(other, bilateral[sid], True)

        return bilateral
            

        
    def postprocess_unionizes(self, raw_unionizes, **kwargs):
        '''Carry out additional calculations/formatting derivative of core 
        unionization.
        
        Parameters
        ----------
        raw_unionizes : list of unionizes
            Each entry is a unionize record.
        
        '''
        raise NotImplementedError('specify in subclass!')
            
            
    def sort_data_arrays(self, data_arrays):
        '''Apply the precomputed sort to flattened data arrays
        
        Parameters
        ----------
        data_arrays : dict
            Keys identify types of data volume. Values are flattened, unsorted 
            arrays.
            
        Returns
        -------
        dict : 
            As input, but values are sorted
        
        '''
        
        logging.info('sorting data arrays')
        return {k: v[self.sort] for k, v in iteritems(data_arrays)}
        
        
    def direct_unionize(self, data_arrays, pre_sorted=False, **kwargs):
        '''Obtain unionize records from directly annotated regions.
        
        Parameters
        ----------
        data_arrays : dict
            Keys identify types of data volume. Values are flattened arrays.
        sorted : bool, optional
            If False, data arrays will be sorted.
        
        '''
        
        if not pre_sorted:
            data_arrays = self.sort_data_arrays(data_arrays)
        
        unionizes = {}
        for sid, (low, high) in iteritems(self.interval_map):
            logging.debug( 'unionizing structure {0} :: voxel_count={1}'.format(sid, high - low) )
            unionizes[sid] = self.extract_data(data_arrays, low, high, **kwargs)
            
        return unionizes



import logging
import os
import sys
import traceback
import logging.config as lc
import shutil
from pkg_resources import resource_filename #@UnresolvedImport
from allensdk.model.biophysical.run_simulate import RunSimulate


class RunSimulateLims(RunSimulate):
    _log = logging.getLogger('allensdk.internal.model.biophysical.run_simulate_lims')

    def __init__(self,
                 input_json, 
                 output_json):
        super(RunSimulateLims, self).__init__(input_json, output_json)

    def generate_manifest_rma(self,
                              neuronal_model_run_id,
                              manifest_path,
                              api_url=None):
        '''
        Note
        ----
        Other necessary files are also written.
        '''
        import json
        from allensdk.internal.api.queries.biophysical_module_api import BiophysicalModuleApi
        from allensdk.internal.api.queries.biophysical_module_reader import BiophysicalModuleReader

        bma = BiophysicalModuleApi(api_url)
        data = bma.get_neuronal_model_runs(neuronal_model_run_id)

        lr = BiophysicalModuleReader()
        lr.read_lims_message(data, 'lims_message.json')

        with open('lims_message.json', 'w') as f:
            f.write(json.dumps(data[0], sort_keys=True, indent=2))

        lr.to_manifest(manifest_path) 

    def generate_manifest_lims(self,
                               lims_data_path,
                               manifest_path):
        '''
        Note
        ----
        Other necessary files are also written.
        '''
        from allensdk.internal.api.queries.biophysical_module_reader import BiophysicalModuleReader

        self.lims_json = lims_data_path

        lr = BiophysicalModuleReader()
        lr.read_lims_file(self.lims_json)
        
        lr.to_manifest(manifest_path)

    def copy_local(self):
        import allensdk.model.biophysical.run_simulate

        self.load_manifest()

        modfile_dir = self.manifest.get_path('MODFILE_DIR')

        if not os.path.exists(modfile_dir):
            os.mkdir(modfile_dir)

        workdir = self.manifest.get_path('WORKDIR')

        if not os.path.exists(workdir):
            os.mkdir(workdir)

        modfiles = [self.manifest.get_path(key) for key,info
                    in self.manifest.path_info.items()
                    if 'format' in info and info['format'] == 'MODFILE']
        
        for from_file in modfiles:             
            RunSimulate._log.debug("copying %s to %s" % (from_file, modfile_dir))
            shutil.copy(from_file, modfile_dir)
        
        shutil.copy(self.manifest.get_path('fit_parameters'),
                    workdir)
        
        shutil.copyfile(self.manifest.get_path('stimulus_path'),
                        self.manifest.get_path('output_path'))
        
        shutil.copy(resource_filename(allensdk.model.biophysical.run_simulate.__name__,
                                      'cell.hoc'),
                    os.curdir)
    
    
def main(command, lims_strategy_json, lims_response_json):
    ''' Entry point for module.
        :param command: select behavior, nrnivmodl or simulate
        :type command: string    
        :param lims_strategy_json: path to json file output from lims.
        :type lims_strategy_json: string
        :param lims_response_json: path to json file returned to lims.
        :type lims_response_json: string
    '''
    rs = RunSimulateLims(lims_strategy_json,
                     lims_response_json)

    RunSimulateLims._log.debug("command: %s" % (command))
    RunSimulateLims._log.debug("lims strategy json: %s" % (lims_strategy_json))
    RunSimulateLims._log.debug("lims upload json: %s" % (lims_response_json))
        
    log_config = resource_filename('allensdk.model.biophysical.run_simulate',
                                    'logging.conf')
    lc.fileConfig(log_config)
    os.environ['LOG_CFG'] = log_config
    
    if 'nrnivmodl' == command:
        rs.nrnivmodl()
    elif 'copy_local' == command:
        rs.copy_local()
    elif 'generate_manifest_rma' == command:
        rs.generate_manifest_rma(input_json, output_json)
    elif 'generate_manifest_lims' == command:
        rs.generate_manifest_lims(input_json, output_json)
    elif 'generate_manifest_lims' == command:
        rs.generate_manifest_lims(input_json, output_json)
    else:
        rs.simulate()


if __name__ == '__main__':
    command, input_json, output_json = sys.argv[-3:]

    try:
        main(command, input_json, output_json)
        RunSimulateLims._log.debug("success")
    except Exception as e:
        RunSimulate._log.error(traceback.format_exc())
        exit(1)



from __future__ import division
import logging

import numpy as np

from .unionize_record import Unionize


class TissuecyteBaseUnionize(Unionize):

    __slots__ = ['sum_pixels', 'sum_projection_pixels', 'sum_projection_pixel_intensity', 
                 'max_voxel_index', 'max_voxel_density', 'projection_density', 
                 'projection_energy', 'projection_intensity', 'direct_sum_projection_pixels', 
                 'sum_pixel_intensity']


    def __init__(self):
        '''A unionize record summarizing observations from a tissuecyte 
        projection experiment
        '''
        
        for key in self.__slots__:
            setattr(self, key, 0)
        
        
    def propagate(self, ancestor, copy_all=False):
        '''Update a rootward unionize with data from this unionize record
        
        Parameters
        ----------
        ancestor : TissuecyteBaseUnionize
            will be updated
            
        Returns
        -------
        ancestor : TissuecyteBaseUnionize
        
        '''
        
        ancestor.sum_pixels += self.sum_pixels
        ancestor.sum_projection_pixels += self.sum_projection_pixels
        ancestor.sum_projection_pixel_intensity += self.sum_projection_pixel_intensity
        ancestor.sum_pixel_intensity += self.sum_pixel_intensity

        if ancestor.max_voxel_density <= self.max_voxel_density:
            ancestor.max_voxel_density = self.max_voxel_density
            ancestor.max_voxel_index = self.max_voxel_index

        if copy_all:
            ancestor.direct_sum_projection_pixels += self.direct_sum_projection_pixels
        
        return ancestor
        
        
    def set_max_voxel(self, density_array, low):
        '''Find the voxel of greatest density in this unionizes spatial domain
        
        Parameters
        ----------
        density_array : ndarray
            Float values are densities per voxel
        low : int
            index in full flattened, sorted array of starting voxel
        
        '''
    
        if self.sum_projection_pixels > 0:
        
            self.max_voxel_index = np.argmax(density_array)
            self.max_voxel_density = density_array[self.max_voxel_index]
            
            self.max_voxel_index += low
            
            
    def output(self, output_spacing_iso, volume_scale, target_shape, sort):
        '''Generate derived data for this unionize
        
        Parameters
        ----------
        output_spacing_iso : numeric
            Isometric spacing of reference space in microns
        volume_scale : numeric
            Scale factor mapping pixels to microns^3
        target_shape : array-like of numeric
            Shape of reference space
        
        '''
        
        if self.sum_pixels > 0:
            self.projection_density = self.sum_projection_pixels / self.sum_pixels
            self.projection_energy = self.sum_projection_pixel_intensity / self.sum_pixels
        
        if self.sum_projection_pixels > 0:
            self.projection_intensity = self.sum_projection_pixel_intensity / self.sum_projection_pixels
        
        output = {k: getattr(self, k) for k in self.__slots__}

        output['volume'] = self.sum_pixels * volume_scale
        output['direct_projection_volume'] = self.direct_sum_projection_pixels * volume_scale
        output['projection_volume'] = self.sum_projection_pixels * volume_scale
        output['sum_pixel_intensity'] = self.sum_pixel_intensity

        if self.max_voxel_index > 0:
            self.max_voxel_index = sort[self.max_voxel_index]
            mv_pos = np.unravel_index([self.max_voxel_index], dims=target_shape, order='C')
            if len(mv_pos[0]) == 0:
                mv_pos = [[0], [0], [0]]
        else:
            mv_pos = [[0], [0], [0]]

        output['max_voxel_x'] = mv_pos[0][0] * output_spacing_iso
        output['max_voxel_y'] = mv_pos[1][0] * output_spacing_iso
        output['max_voxel_z'] = mv_pos[2][0] * output_spacing_iso
        del output['max_voxel_index']
        
        return output
        
        
class TissuecyteInjectionUnionize(TissuecyteBaseUnionize):
    
    def calculate(self, low, high, data_arrays):
        data_arrays = self.slice_arrays(low, high, data_arrays)
        
        self.sum_pixels = np.multiply(data_arrays['sum_pixels'], data_arrays['injection_fraction']).sum()
        self.sum_projection_pixels = np.multiply(data_arrays['sum_pixels'], data_arrays['injection_density']).sum()
        self.direct_sum_projection_pixels = self.sum_projection_pixels
        self.sum_projection_pixel_intensity = np.multiply(data_arrays['sum_pixels'], data_arrays['injection_energy']).sum()
        self.sum_pixel_intensity = data_arrays['injection_sum_pixel_intensities'].sum()

        self.set_max_voxel(data_arrays['injection_density'], low)
            
            
class TissuecyteProjectionUnionize(TissuecyteBaseUnionize):
    
    def calculate(self, low, high, data_arrays, ij_record):
        data_arrays = self.slice_arrays(low, high, data_arrays)
        
        nex = np.logical_or(data_arrays['injection_fraction'], np.logical_not(data_arrays['aav_exclusion_fraction']))
        
        self.sum_pixels = data_arrays['sum_pixels'][nex].sum()
        self.sum_pixels -= ij_record.sum_pixels
        
        self.sum_projection_pixels = np.multiply(data_arrays['sum_pixels'], data_arrays['projection_density'])[nex].sum()
        self.sum_projection_pixels -= ij_record.sum_projection_pixels
        self.direct_sum_projection_pixels = self.sum_projection_pixels
        
        self.sum_projection_pixel_intensity = np.multiply(data_arrays['sum_pixels'], data_arrays['projection_energy'])[nex].sum()
        self.sum_projection_pixel_intensity -= ij_record.sum_projection_pixel_intensity

        self.sum_pixel_intensity = float(data_arrays['sum_pixel_intensities'][nex].sum())
        self.sum_pixel_intensity -= ij_record.sum_pixel_intensity
        
        valid_density = np.multiply(nex, data_arrays['projection_density'])
        valid_density = np.multiply(valid_density, 1 - data_arrays['injection_fraction']) 
        self.set_max_voxel(valid_density, low)



import logging
import numpy as np
import h5py

#import matplotlib
#matplotlib.use('agg')
import matplotlib.pyplot as plt
import os

def background_trace(trace, save_dir, data_set=None):

    fig,ax = plt.subplots(1)
    ax.plot(trace)
    if data_set is not None:
        _add_stim_epochs(trace, ax, data_set)

    save_file = os.path.join(save_dir, 'background_trace.pdf')
    fig.savefig(save_file)
    logging.info("Background Trace saved to %s", save_file)

    plt.close(fig)

def correlation_report(dm, save_dir, without_masks=True):
    '''
        parameters:
            dm:  [DeMix object]
            without_masks:  boolean
    '''

    logging.info("Generating Correlation Report")
    if without_masks:
        cor, cor_demix = compute_correlations_without_masks(dm)

        no_diagonal_mask = (1.0 - np.eye(cor.shape[0])).astype('bool')

        fig, ax = plt.subplots(1)
        ax.plot(dm.mask_overlap[:-1,:-1][no_diagonal_mask],cor[no_diagonal_mask]-cor_demix[no_diagonal_mask],'o')
        ax.set_xlim([-1,np.max(dm.mask_overlap[:-1,:-1][no_diagonal_mask])])
        ax.set_title('Delta Correlation vs. mask overlap')

        save_file = os.path.join(save_dir,'cor_vs_overlap.pdf')
        fig.savefig(save_file)
        logging.info("\tCorrelation overlap saved to %s", save_file)
        plt.close(fig)

        fig, ax = plt.subplots(1,3)
        delta_cor = cor- cor_demix
        ax[0].hist(delta_cor[dm.mask_overlap[:-1,:-1]==0],bins=100)
        ax[0].set_title('Delta Correlation')
        ax[1].hist(cor[no_diagonal_mask],bins=100)
        ax[1].set_title('Pre-demix Correlation')
        ax[2].hist(cor_demix[no_diagonal_mask],bins=100)
        ax[2].set_title('Post-demix Correlation')

        save_file = os.path.join(save_dir,'cor_hist.pdf')
        fig.savefig(save_file)
        logging.info("\tCorrelation histograms saved to %s", save_file)
        plt.close(fig)


    else:
        raise Exception('without_masks=False not yet implemented')

def plot_masks(dm, save_dir, movie_file, movie_dataset, window=150, add_background=True):

    logging.info("Plotting masks")

    overlap_pairs =  [(x,y) for (x,y) in  zip(*np.where(dm.mask_overlap >0)) if x>y and x!=dm.mask_overlap.shape[0]-1]
    movie_data = h5py.File(movie_file,'r')

    bg_traces = dm.get_traces_with_background()

    for i,pair in enumerate(overlap_pairs):
        fig_pair, ax_pair = plt.subplots(2,2)
        
        mask = np.zeros(dm.masks[0].shape)
        rgb_shape = (dm.masks[0].shape[0],dm.masks[0].shape[1],3)
        rgb_mask = np.zeros(rgb_shape,dtype=np.uint8)
        #for p in pair:
        rgb_mask[:,:,2] = 255*dm.masks[pair[0]]
        rgb_mask[:,:,1] = 255*dm.masks[pair[1]]
        for p in pair:
            mask += dm.masks[p]
        non_zeros = np.where(mask)
        non_zeros_mask = np.zeros(mask.shape)
        ylower = np.min(non_zeros[0])
        yupper = np.max(non_zeros[0])
        xlower = np.min(non_zeros[1])
        xupper = np.max(non_zeros[1])

        #ax_pair[0,0].imshow(mask[ylower:yupper,xlower:xupper])
        ax_pair[0,0].imshow(rgb_mask[ylower:yupper,xlower:xupper])

        trace1 = dm.traces[pair[0]]
        trace2 = dm.traces[pair[1]]

        if add_background:
            trace1_demix = bg_traces[pair[0]]
            trace2_demix = bg_traces[pair[1]]
        else:
            trace1_demix = dm.traces_demix[pair[0]]
            trace2_demix = dm.traces_demix[pair[1]]

        center = np.where(trace1==np.max(trace1))[0][0]

        ax_pair[1,0].plot(trace1[(center-window):(center+window)],label=str(pair[0]))
        ax_pair[1,0].plot(trace2[(center-window):(center+window)],label=str(pair[1]))
        ax_pair[1,0].legend()

        ax_pair[1,1].plot(trace1_demix[center-window:center+window],label=str(pair[0]))
        ax_pair[1,1].plot(trace2_demix[center-window:center+window],label=str(pair[1]))
        ax_pair[1,1].legend()

        ax_pair[0,1].imshow(movie_data[movie_dataset][center,ylower:yupper,xlower:xupper])

        save_file = os.path.join(save_dir,'masks_'+str(pair[0])+'_'+str(pair[1])+'.pdf')
        fig_pair.savefig(save_file)
        plt.close(fig_pair)
        logging.info("\tMask saved to %s", save_file)

    #print overlap_pairs
    #print np.unique(dm.mask_overlap[no_diagonal_mask][dm.mask_overlap[no_diagonal_mask]>0])


def _get_epoch_windows(stim_table):

    start = np.array(stim_table.start)
    end = np.array(stim_table.end)

    windows = zip(start,end)
    window_list = [[start[0]]]
    for i,w in enumerate(windows[1:]):
        if start[i+1] - end[i]>1:
            window_list[-1].append(end[i])
            window_list.append([start[i+1]])
            #window_list += [end[i],start[i+1]] 

    window_list[-1].append(end[-1])

    #window_list = [start[0]]
    #window_list += [ start[x+1] for x in  list(np.where(np.abs(start[1:] - end[:-1]) > 1)[0])]
    #window_list += [end[-1]]

    #print window_list

    return window_list

def _add_stim_epochs(trace,ax,data_set):

    stim_colors_dict = {'locally_sparse_noise':'green','drifting_gratings':'yellow','natural_movie_one':'magenta','natural_movie_two':'magenta','natural_movie_three':'red','natural_scenes':'orange','spontaneous':'grey','static_gratings':'blue'}

    from allensdk.brain_observatory.stimulus_info import stimuli_in_session

    stim_types = stimuli_in_session(data_set.get_metadata()['session_type'])


    for stim in stim_types:
        #print stim
        stim_table = data_set.get_stimulus_table(stim)
        window_list = _get_epoch_windows(stim_table)
        for w in window_list:
            #ax.fill_betweenx(np.arange(trace.shape[0]),w[0],w[1],facecolor=stim_colors_dict[stim],alpha=0.2)
            #ax.axvspan(w[0],w[1],np.min(trace),np.max(trace),facecolor=stim_colors_dict[stim],alpha=0.2)
            ax.axvspan(w[0],w[1],facecolor=stim_colors_dict[stim],alpha=0.2)
            ax.set_ylim([np.min(trace),np.max(trace)])


def compute_non_overlap_masks(dm):

    no_masks = np.zeros(dm.masks.shape).astype(int)
    overlap_val = np.zeros(dm.masks.shape[0])

    for i, m in enumerate(dm.masks):

        overlap_1 = np.sum(dm.masks[:i],axis=0)
        overlap_2 = np.sum(dm.masks[i+1:],axis=0)

        overlap = overlap_1 + overlap_2

        overlap_val[i] = np.sum(overlap)

        no1 = overlap == 0
        #no2 = overlap_2 == 0

        no_masks[i] = np.logical_and(no1, m)

    dm.no_masks = no_masks
    dm.overlap = overlap

    return dm.no_masks

def compute_non_overlap_traces(dm, movie_path, movie_dataset):
    no_traces_shape = (dm.traces.shape[0],dm.traces.shape[1])
    no_traces = np.zeros(no_traces_shape)

    N, T = no_traces.shape

    chunk_size = 1000
    num_chunks = int(np.ceil(T/float(chunk_size))) 

    normalized_flat_masks = dm.no_masks.reshape(N,-1).T          # shape (pixels, N)
    normalized_flat_masks /= np.sum(normalized_flat_masks,axis=0)  # shape(pixels, N)

    movie_f = h5py.File(movie_path)
    movie = movie_f[movie_dataset]

    logging.debug("Getting traces for %d chunks", num_chunks)
    for n in range(num_chunks):
        print "Chunk = ", n
        data = movie[n*chunk_size:(n+1)*chunk_size]
        data = data.reshape(chunk_size,-1)  # This line throws an error

        no_traces[:,n*chunk_size:(n+1)*chunk_size] = np.dot(data,normalized_flat_masks).T

    movie_f.close()

    logging.debug("Done")
    dm.no_traces = no_traces

    return dm.no_traces

def compute_correlations(dm, movie_path, movie_dataset):

    compute_non_overlap_masks(dm)
    compute_non_overlap_traces(dm, movie_path, movie_dataset)

    no_mean = np.mean(dm.no_traces)
    dm_mean = np.mean(dm.traces_demix)
    t_mean = np.mean(dm.traces)

    C_no_dm = np.mean( (dm.no_traces-no_mean)*(dm.traces_demix-dm_mean), axis=1)
    C_t_dm = np.mean( (dm.traces-t_mean)*(dm.traces_demix-dm_mean), axis=1)

    return C_no_dm, C_t_dm

def compute_correlations_without_masks(dm):
    N, T = dm.traces.shape
    N=N -1

    traces = (dm.traces.T - np.mean(dm.traces.T,axis=0))  # shape (T,N)
    traces_demix = (dm.traces_demix.T - np.mean(dm.traces_demix.T,axis=0))  # shape (T,N)

    traces /= np.std(traces,axis=0)
    traces_demix /= np.std(traces_demix,axis=0)

    cor = np.dot(traces.T,traces)/T
    cor_demix = np.dot(traces_demix.T,traces_demix)/T

    return cor[:N,:N], cor_demix[:N,:N]




import logging

import SimpleITK as sitk
import numpy as np

import volume_utilities as vol


class VolumeProjector(object):

    def __init__(self, view_volume):
        logging.info('initializing volume projector')
        self.view_volume = view_volume
        

    def build_rotation_transform(self, from_axis, to_axis, angle):
        logging.info('constructing rotation')        

        transform = sitk.AffineTransform(3)
        transform.SetCenter((vol.sitk_get_center(self.view_volume)).tolist())
        transform.Rotate(to_axis, from_axis, angle, True)

        logging.info(transform.__str__())
        return transform

        
    def rotate(self, from_axis, to_axis, angle):
        logging.info('rotating from axis {0} to axis {1} '
                     'by {2:2.2f} radians'.format(from_axis, to_axis, angle))
        
        transform = self.build_rotation_transform(from_axis, to_axis, angle)
        rotated = sitk.Resample(self.view_volume, transform, sitk.sitkLinear, 
                                0.0, self.view_volume.GetPixelID())

        return rotated
    
    
    def extract(self, cb, volume=None):
        logging.info('extracting projection')
        
        if volume is None:
            volume=self.view_volume

        return cb(volume)

    
    def rotate_and_extract(self, from_axes, to_axes, angles, cb):
        
        for fax, tax, angle in zip(from_axes, to_axes, angles):
                
            rotated = self.rotate(fax, tax, angle)
            yield self.extract(cb, rotated)
    

    @classmethod
    def fixed_factory(cls, volume, size):

        view_volume = sitk.Image(size[0], size[1], size[2], volume.GetPixelID())
        view_volume = vol.sitk_paste_into_center(volume, view_volume)

        return cls(view_volume)


    @classmethod
    def safe_factory(cls, volume):

        max_extent = vol.sitk_get_diagonal_length(volume)
        max_extent = [np.ceil(max_extent).astype(int)] * 3

        vpar = vol.sitk_get_size_parity(volume)
        lpar = np.mod(max_extent, 2)

        for ax in xrange(volume.GetDimension()):
            if vpar[ax] != lpar[ax]:
                max_extent[ax] += 1
        
        return cls.fixed_factory(volume, max_extent)





